@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -12,19 +10,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
+"""
 A middleware that turns exceptions into parsable string.
-'''
+"""
 
-import traceback
-
-from oslo_config import cfg
+from oslo_utils import reflection
 import six
 import webob
 
+from senlin.api.common import serializers
+from senlin.api.common import wsgi
 from senlin.common import exception
-from senlin.common import serializers
-from senlin.common import wsgi
 
 
 class Fault(object):
@@ -47,31 +43,19 @@ class FaultWrapper(wsgi.Middleware):
 
     error_map = {
         'ActionInProgress': webob.exc.HTTPConflict,
-        'ActionNotFound': webob.exc.HTTPNotFound,
-        'ClusterNotFound': webob.exc.HTTPNotFound,
-        'EventNotFound': webob.exc.HTTPNotFound,
+        'BadRequest': webob.exc.HTTPBadRequest,
         'FeatureNotSupported': webob.exc.HTTPConflict,
         'Forbidden': webob.exc.HTTPForbidden,
         'InternalError': webob.exc.HTTPInternalServerError,
-        'InvalidParameter': webob.exc.HTTPBadRequest,
-        'InvalidSchemaError': webob.exc.HTTPBadRequest,
+        'InvalidGlobalAPIVersion': webob.exc.HTTPNotAcceptable,
+        'InvalidSpec': webob.exc.HTTPBadRequest,
+        'MethodVersionNotFound': webob.exc.HTTPBadRequest,
         'MultipleChoices': webob.exc.HTTPBadRequest,
-        'NodeNotFound': webob.exc.HTTPNotFound,
         'NodeNotOrphan': webob.exc.HTTPConflict,
         'PolicyBindingNotFound': webob.exc.HTTPNotFound,
-        'PolicyNotFound': webob.exc.HTTPNotFound,
-        'PolicyTypeNotFound': webob.exc.HTTPNotFound,
-        'ProfileInUse': webob.exc.HTTPConflict,
-        'ProfileNotFound': webob.exc.HTTPNotFound,
-        'ProfileTypeNotFound': webob.exc.HTTPNotFound,
-        'ProfileTypeNotMatch': webob.exc.HTTPBadRequest,
         'RequestLimitExceeded': webob.exc.HTTPBadRequest,
         'ResourceInUse': webob.exc.HTTPConflict,
-        'SenlinBadRequest': webob.exc.HTTPBadRequest,
-        'SpecValidationFailed': webob.exc.HTTPBadRequest,
-        'TriggerNotFound': webob.exc.HTTPNotFound,
-        'TriggerTypeNotFound': webob.exc.HTTPNotFound,
-        'WebhookNotFound': webob.exc.HTTPNotFound,
+        'ResourceNotFound': webob.exc.HTTPNotFound,
     }
 
     def _map_exception_to_error(self, class_exception):
@@ -84,18 +68,13 @@ class FaultWrapper(wsgi.Middleware):
         return self.error_map[class_exception.__name__]
 
     def _error(self, ex):
-        trace = None
         traceback_marker = 'Traceback (most recent call last)'
         webob_exc = None
         if isinstance(ex, exception.HTTPExceptionDisguise):
-            # An HTTP exception was disguised so it could make it here
-            # let's remove the disguise and set the original HTTP exception
-            if cfg.CONF.debug:
-                trace = ''.join(traceback.format_tb(ex.tb))
             ex = ex.exc
             webob_exc = ex
 
-        ex_type = ex.__class__.__name__
+        ex_type = reflection.get_class_name(ex, fully_qualified=False)
 
         is_remote = ex_type.endswith('_Remote')
         if is_remote:
@@ -103,26 +82,16 @@ class FaultWrapper(wsgi.Middleware):
 
         full_message = six.text_type(ex)
         if '\n' in full_message and is_remote:
-            message, msg_trace = full_message.split('\n', 1)
+            message = full_message.split('\n', 1)[0]
         elif traceback_marker in full_message:
-            message, msg_trace = full_message.split(traceback_marker, 1)
+            message = full_message.split(traceback_marker, 1)[0]
             message = message.rstrip('\n')
-            msg_trace = traceback_marker + msg_trace
         else:
-            if six.PY3:
-                msg_trace = traceback.format_exception(type(ex), ex,
-                                                       ex.__traceback__)
-            else:
-                msg_trace = traceback.format_exc()
-
             message = full_message
 
         if isinstance(ex, exception.SenlinException):
             message = ex.message
 
-        if cfg.CONF.debug and not trace:
-            trace = msg_trace
-
         if not webob_exc:
             webob_exc = self._map_exception_to_error(ex.__class__)
 
@@ -134,7 +103,6 @@ class FaultWrapper(wsgi.Middleware):
                 'code': webob_exc.code,
                 'message': message,
                 'type': ex_type,
-                'traceback': trace,
             }
         }
 
