@@ -13,21 +13,14 @@
 
 from webob import exc
 
-from oslo_log import log as logging
-
-from senlin.api.openstack.v1 import util
+from senlin.api.common import util
+from senlin.api.common import wsgi
 from senlin.common import consts
 from senlin.common.i18n import _
-from senlin.common import serializers
-from senlin.common import utils
-from senlin.common import wsgi
-from senlin.rpc import client as rpc_client
-
-LOG = logging.getLogger(__name__)
 
 
 class ActionData(object):
-    '''All required data fields for an action.'''
+    """All required data fields for an action."""
 
     PARAMS = (consts.ACTION_NAME, consts.ACTION_TARGET, consts.ACTION_ACTION)
 
@@ -54,53 +47,36 @@ class ActionData(object):
         return dict((k, v) for k, v in data if k not in self.PARAMS)
 
 
-class ActionController(object):
-    '''WSGI controller for Actions in Senlin v1 API.'''
+class ActionController(wsgi.Controller):
+    """WSGI controller for Actions in Senlin v1 API."""
 
     # Define request scope (must match what is in policy.json)
     REQUEST_SCOPE = 'actions'
 
-    def __init__(self, options):
-        self.options = options
-        self.rpc_client = rpc_client.EngineClient()
-
-    def default(self, req, **args):
-        raise exc.HTTPNotFound()
-
     @util.policy_enforce
     def index(self, req):
-        filter_whitelist = {
-            'name': 'mixed',
-            'target': 'mixed',
-            'action': 'mixed',
-            'created_time': 'single',
-            'updated_time': 'single',
-            'deleted_time': 'single',
+        whitelist = {
+            consts.ACTION_NAME: 'mixed',
+            consts.ACTION_TARGET: 'mixed',
+            consts.ACTION_ACTION: 'mixed',
+            consts.ACTION_STATUS: 'mixed',
+            consts.PARAM_LIMIT: 'single',
+            consts.PARAM_MARKER: 'single',
+            consts.PARAM_SORT: 'single',
+            consts.PARAM_GLOBAL_PROJECT: 'single',
         }
-        param_whitelist = {
-            'limit': 'single',
-            'marker': 'single',
-            'sort_dir': 'single',
-            'sort_keys': 'multi',
-            'show_deleted': 'single',
-        }
-        params = util.get_allowed_params(req.params, param_whitelist)
-        filters = util.get_allowed_params(req.params, filter_whitelist)
+        for key in req.params.keys():
+            if key not in whitelist.keys():
+                raise exc.HTTPBadRequest(_('Invalid parameter %s') % key)
+        params = util.get_allowed_params(req.params, whitelist)
 
-        key = consts.PARAM_LIMIT
-        if key in params:
-            params[key] = utils.parse_int_param(key, params[key])
+        project_safe = not util.parse_bool_param(
+            consts.PARAM_GLOBAL_PROJECT,
+            params.pop(consts.PARAM_GLOBAL_PROJECT, False))
+        params['project_safe'] = project_safe
 
-        key = consts.PARAM_SHOW_DELETED
-        if key in params:
-            params[key] = utils.parse_bool_param(key, params[key])
-
-        if not filters:
-            filters = None
-
-        actions = self.rpc_client.action_list(req.context,
-                                              filters=filters,
-                                              **params)
+        obj = util.parse_request('ActionListRequest', req, params)
+        actions = self.rpc_client.call(req.context, "action_list", obj)
 
         return {'actions': actions}
 
@@ -117,16 +93,8 @@ class ActionController(object):
 
     @util.policy_enforce
     def get(self, req, action_id):
-        action = self.rpc_client.action_get(req.context, action_id)
-        if not action:
-            raise exc.HTTPNotFound()
+        params = {'identity': action_id}
+        obj = util.parse_request('ActionGetRequest', req, params)
+        action = self.rpc_client.call(req.context, 'action_get', obj)
 
         return {'action': action}
-
-
-def create_resource(options):
-    '''Actions factory method.'''
-
-    return wsgi.Resource(ActionController(options),
-                         wsgi.JSONRequestDeserializer(),
-                         serializers.JSONResponseSerializer())
