@@ -23,35 +23,35 @@ import sys
 from oslo_log import log as logging
 import six
 
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
+from senlin.common.i18n import _, _LE
 
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 LOG = logging.getLogger(__name__)
 
 
 class SenlinException(Exception):
-    '''Base Senlin Exception.
-
-    To correctly use this class, inherit from it and define
-    a 'msg_fmt' property. That msg_fmt will get printf'd
-    with the keyword arguments provided to the constructor.
-    '''
+    """Base Senlin Exception.
 
+    To correctly use this class, inherit from it and define a 'msg_fmt'
+    property. That msg_fmt will get printed with the keyword arguments
+    provided to the constructor.
+    """
     message = _("An unknown exception occurred.")
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
-        self.__context__ = None
 
         try:
             self.message = self.msg_fmt % kwargs
+            # if last char is '.', wipe out redundant '.'
+            if self.message[-1] == '.':
+                self.message = self.message.rstrip('.') + '.'
         except KeyError:
             # exc_info = sys.exc_info()
             # if kwargs doesn't match a variable in the message
             # log the issue and the kwargs
             LOG.exception(_LE('Exception in string format operation'))
-            for name, value in six.iteritems(kwargs):
+            for name, value in kwargs.items():
                 LOG.error("%s: %s" % (name, value))  # noqa
 
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
@@ -77,40 +77,55 @@ class NotAuthenticated(SenlinException):
 
 
 class Forbidden(SenlinException):
-    msg_fmt = _("You are not authorized to complete this action.")
+    msg_fmt = _("You are not authorized to complete this operation.")
 
 
-class SenlinBadRequest(SenlinException):
-    msg_fmt = _("The request is malformed: %(msg)s")
+class BadRequest(SenlinException):
+    msg_fmt = _("%(msg)s.")
+
+
+class InvalidAPIVersionString(SenlinException):
+    msg_fmt = _("API Version String '%(version)s' is of invalid format. It "
+                "must be of format 'major.minor'.")
+
+
+class MethodVersionNotFound(SenlinException):
+    msg_fmt = _("API version '%(version)s' is not supported on this method.")
+
+
+class InvalidGlobalAPIVersion(SenlinException):
+    msg_fmt = _("Version '%(req_ver)s' is not supported by the API. Minimum "
+                "is '%(min_ver)s' and maximum is '%(max_ver)s'.")
 
 
 class MultipleChoices(SenlinException):
-    msg_fmt = _("Multiple results found matching the query criteria %(arg)s. "
-                "Please be more specific.")
+    msg_fmt = _("Multiple results found matching the query criteria "
+                "'%(arg)s'. Please be more specific.")
 
 
-class InvalidParameter(SenlinException):
-    msg_fmt = _("Invalid value '%(value)s' specified for '%(name)s'")
+class ResourceNotFound(SenlinException):
+    """Generic exception for resource not found.
+
+    The resource type here can be 'cluster', 'node', 'profile',
+    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
+    'action', 'event' and so on.
+    """
+    msg_fmt = _("The %(type)s '%(id)s' could not be found.")
+
+    @staticmethod
+    def enhance_msg(enhance, ex):
+        enhance_msg = ex.message[:4] + enhance + ' ' + ex.message[4:]
+        return enhance_msg
 
 
-class ClusterNotFound(SenlinException):
-    msg_fmt = _("The cluster (%(cluster)s) could not be found.")
+class ResourceInUse(SenlinException):
+    """Generic exception for resource in use.
 
-
-class NodeNotFound(SenlinException):
-    msg_fmt = _("The node (%(node)s) could not be found.")
-
-
-class ProfileTypeNotFound(SenlinException):
-    msg_fmt = _("Profile type (%(profile_type)s) is not found.")
-
-
-class ProfileTypeNotMatch(SenlinException):
-    msg_fmt = _("%(message)s")
-
-
-class ProfileNotFound(SenlinException):
-    msg_fmt = _("The profile (%(profile)s) could not be found.")
+    The resource type here can be 'cluster', 'node', 'profile',
+    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
+    'action', 'event' and so on.
+    """
+    msg_fmt = _("The %(type)s '%(id)s' cannot be deleted: %(reason)s.")
 
 
 class ProfileNotSpecified(SenlinException):
@@ -129,39 +144,19 @@ class PolicyNotSpecified(SenlinException):
     msg_fmt = _("Policy not specified.")
 
 
-class PolicyTypeNotFound(SenlinException):
-    msg_fmt = _("Policy type (%(policy_type)s) is not found.")
-
-
-class PolicyNotFound(SenlinException):
-    msg_fmt = _("The policy (%(policy)s) could not be found.")
-
-
 class PolicyBindingNotFound(SenlinException):
-    msg_fmt = _("The policy (%(policy)s) is not found attached to the "
-                "specified cluster (%(identity)s).")
+    msg_fmt = _("The policy '%(policy)s' is not found attached to the "
+                "specified cluster '%(identity)s'.")
 
 
 class PolicyTypeConflict(SenlinException):
-    msg_fmt = _("The policy with type (%(policy_type)s) already exists.")
+    msg_fmt = _("The policy with type '%(policy_type)s' already exists.")
 
 
-class TriggerTypeNotFound(SenlinException):
-    msg_fmt = _("Trigger type (%(trigger_type)s) is not found.")
-
-
-class InvalidSchemaError(SenlinException):
+class InvalidSpec(SenlinException):
     msg_fmt = _("%(message)s")
 
 
-class SpecValidationFailed(SenlinException):
-    msg_fmt = _("%(message)s")
-
-
-class TriggerNotFound(SenlinException):
-    msg_fmt = _("The trigger (%(trigger)s) could not be found.")
-
-
 class FeatureNotSupported(SenlinException):
     msg_fmt = _("%(feature)s is not supported.")
 
@@ -173,10 +168,6 @@ class Error(SenlinException):
         super(Error, self).__init__(message=msg)
 
 
-class ResourceInUse(SenlinException):
-    msg_fmt = _("The %(resource_type)s (%(resource_id)s) is still in use.")
-
-
 class InvalidContentType(SenlinException):
     msg_fmt = _("Invalid content type %(content_type)s")
 
@@ -185,21 +176,8 @@ class RequestLimitExceeded(SenlinException):
     msg_fmt = _('Request limit exceeded: %(message)s')
 
 
-class WebhookNotFound(SenlinException):
-    msg_fmt = _("The webhook (%(webhook)s) could not be found.")
-
-
-class ActionNotFound(SenlinException):
-    msg_fmt = _("The action (%(action)s) could not be found.")
-
-
 class ActionInProgress(SenlinException):
-    msg_fmt = _("Cluster %(cluster_name)s already has an action (%(action)s) "
-                "in progress.")
-
-
-class EventNotFound(SenlinException):
-    msg_fmt = _("The event (%(event)s) could not be found.")
+    msg_fmt = _("The %(type)s '%(id)s' is in status %(status)s.")
 
 
 class NodeNotOrphan(SenlinException):
@@ -207,67 +185,79 @@ class NodeNotOrphan(SenlinException):
 
 
 class InternalError(SenlinException):
-    '''A base class for internal exceptions in senlin.
+    """A base class for internal exceptions in senlin.
 
-    The internal exception classes which inherit from :class:`InternalError`
+    The internal exception classes which inherit from :class:`SenlinException`
     class should be translated to a user facing exception type if need to be
     made user visible.
-    '''
-    msg_fmt = _('ERROR %(code)s happens for %(message)s.')
-    message = _('Internal error happens')
+    """
+    msg_fmt = _("%(message)s")
+    message = _('Internal error happened')
 
     def __init__(self, **kwargs):
-        super(InternalError, self).__init__(**kwargs)
-        if 'code' in kwargs.keys():
-            self.code = kwargs.get('code', 500)
-            self.message = kwargs.get('message')
+        self.code = kwargs.pop('code', 500)
+        self.message = kwargs.pop('message', self.message)
+        super(InternalError, self).__init__(
+            code=self.code, message=self.message, **kwargs)
 
 
-class ResourceBusyError(InternalError):
-    msg_fmt = _("The %(resource_type)s (%(resource_id)s) is busy now.")
+class EResourceBusy(InternalError):
+    # Internal exception, not to be exposed to end user.
+    msg_fmt = _("The %(type)s '%(id)s' is busy now.")
 
 
 class TrustNotFound(InternalError):
     # Internal exception, not to be exposed to end user.
-    msg_fmt = _("The trust for trustor (%(trustor)s) could not be found.")
+    msg_fmt = _("The trust for trustor '%(trustor)s' could not be found.")
 
 
-class ResourceCreationFailure(InternalError):
+class EResourceCreation(InternalError):
     # Used when creating resources in other services
-    msg_fmt = _("Failed in creating %(rtype)s.")
+    def __init__(self, **kwargs):
+        self.resource_id = kwargs.pop('resource_id', None)
+        super(EResourceCreation, self).__init__(
+            resource_id=self.resource_id, **kwargs)
+    msg_fmt = _("Failed in creating %(type)s: %(message)s.")
 
 
-class ResourceUpdateFailure(InternalError):
+class EResourceUpdate(InternalError):
     # Used when updating resources from other services
-    msg_fmt = _("Failed in updating %(resource)s.")
+    msg_fmt = _("Failed in updating %(type)s '%(id)s': %(message)s.")
 
 
-class ResourceDeletionFailure(InternalError):
+class EResourceDeletion(InternalError):
     # Used when deleting resources from other services
-    msg_fmt = _("Failed in deleting %(resource)s.")
+    msg_fmt = _("Failed in deleting %(type)s '%(id)s': %(message)s.")
 
 
-class ResourceNotFound(InternalError):
-    # Used when retrieving resources from other services
-    msg_fmt = _("The resource (%(resource)s) could not be found.")
+class EResourceOperation(InternalError):
+    """Generic exception for resource fail operation.
+
+    The op here can be 'recovering','rebuilding', 'checking' and
+    so on. And the op 'creating', 'updating' and 'deleting' we can
+    use separately class `EResourceCreation`,`EResourceUpdate` and
+    `EResourceDeletion`.
+    The type here is resource's driver type.It can be 'server',
+    'stack', 'container' and so on.
+    The id is resource's id.
+    The message here can be message from class 'ResourceNotFound',
+    'ResourceInUse' and so on, or developer can specified message.
+    """
+    # Used when operating resources from other services
+    msg_fmt = _("Failed in %(op)s %(type)s '%(id)s': %(message)s.")
 
 
-class ResourceStatusError(InternalError):
-    msg_fmt = _("The resource %(resource_id)s is in error status "
-                "- '%(status)s' due to '%(reason)s'.")
+class ESchema(InternalError):
+    msg_fmt = _("%(message)s")
 
 
 class InvalidPlugin(InternalError):
     msg_fmt = _("%(message)s")
 
 
-class InvalidSpec(InternalError):
-    msg_fmt = _("%(message)s")
-
-
 class PolicyNotAttached(InternalError):
-    msg_fmt = _("The policy (%(policy)s) is not attached to the specified "
-                "cluster (%(cluster)s).")
+    msg_fmt = _("The policy '%(policy)s' is not attached to the specified "
+                "cluster '%(cluster)s'.")
 
 
 class HTTPExceptionDisguise(Exception):
