@@ -17,20 +17,19 @@ from oslo_context import context as oslo_context
 from oslo_log import log as logging
 
 from senlin.common import exception
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
+from senlin.common.i18n import _, _LE
 from senlin.drivers import base
 from senlin.drivers.openstack import neutron_v2 as neutronclient
-from senlin.engine import event
 
 LOG = logging.getLogger(__name__)
 
 
 class LoadBalancerDriver(base.DriverBase):
-    """Load-balancing driver based on Neutron LBaaS service."""
+    """Load-balancing driver based on Neutron LBaaS V2 service."""
 
     def __init__(self, params):
         super(LoadBalancerDriver, self).__init__(params)
+        self.lb_status_timeout = 600
         self._nc = None
 
     def nc(self):
@@ -40,7 +39,7 @@ class LoadBalancerDriver(base.DriverBase):
         self._nc = neutronclient.NeutronClient(self.conn_params)
         return self._nc
 
-    def _wait_for_lb_ready(self, lb_id, timeout=60, ignore_not_found=False):
+    def _wait_for_lb_ready(self, lb_id, ignore_not_found=False):
         """Keep waiting until loadbalancer is ready
 
         This method will keep waiting until loadbalancer resource specified
@@ -48,20 +47,17 @@ class LoadBalancerDriver(base.DriverBase):
         its operating_status is ONLINE.
 
         :param lb_id: ID of the load-balancer to check.
-        :param timeout: timeout in seconds.
         :param ignore_not_found: if set to True, nonexistent loadbalancer
             resource is also an acceptable result.
         """
         waited = 0
-        while waited < timeout:
+        while waited < self.lb_status_timeout:
             try:
                 lb = self.nc().loadbalancer_get(lb_id)
             except exception.InternalError as ex:
                 msg = _LE('Failed in getting loadbalancer: %s.'
                           ) % six.text_type(ex)
                 LOG.exception(msg)
-                event.warning(oslo_context.get_current(), self, 'LB_GET',
-                              'ERROR', msg)
                 return False
             if lb is None:
                 lb_ready = ignore_not_found
@@ -72,19 +68,20 @@ class LoadBalancerDriver(base.DriverBase):
             if lb_ready is True:
                 return True
 
-            LOG.debug(_('Waiting for loadbalancer %(lb)s to become ready'),
+            LOG.debug('Waiting for loadbalancer %(lb)s to become ready',
                       {'lb': lb_id})
 
-            eventlet.sleep(2)
-            waited += 2
+            eventlet.sleep(10)
+            waited += 10
 
         return False
 
-    def lb_create(self, vip, pool):
+    def lb_create(self, vip, pool, hm=None):
         """Create a LBaaS instance
 
         :param vip: A dict containing the properties for the VIP;
         :param pool: A dict describing the pool of load-balancer members.
+        :param hm: A dict describing the health monitor.
         """
         def _cleanup(msg, **kwargs):
             LOG.error(msg)
@@ -98,8 +95,6 @@ class LoadBalancerDriver(base.DriverBase):
         except exception.InternalError as ex:
             msg = _LE('Failed in getting subnet: %s.') % six.text_type(ex)
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self, 'SUBNET_GET',
-                          'ERROR', msg)
             return False, msg
         subnet_id = subnet.id
         try:
@@ -110,14 +105,14 @@ class LoadBalancerDriver(base.DriverBase):
             msg = _LE('Failed in creating loadbalancer: %s.'
                       ) % six.text_type(ex)
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self, 'LB_CREATE',
-                          'ERROR', msg)
             return False, msg
         result['loadbalancer'] = lb.id
+        result['vip_address'] = lb.vip_address
 
         res = self._wait_for_lb_ready(lb.id)
         if res is False:
-            msg = _LE('Failed in creating load balancer (%s).') % lb.id
+            msg = _LE('Failed in creating loadbalancer (%s).') % lb.id
+            del result['vip_address']
             _cleanup(msg, **result)
             return False, msg
 
@@ -132,13 +127,12 @@ class LoadBalancerDriver(base.DriverBase):
             msg = _LE('Failed in creating lb listener: %s.'
                       ) % six.text_type(ex)
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self, 'LISTENER_CREATE',
-                          'ERROR', msg)
             return False, msg
         result['listener'] = listener.id
         res = self._wait_for_lb_ready(lb.id)
         if res is False:
             msg = _LE('Failed in creating listener (%s).') % listener.id
+            del result['vip_address']
             _cleanup(msg, **result)
             return res, msg
 
@@ -151,13 +145,38 @@ class LoadBalancerDriver(base.DriverBase):
             msg = _LE('Failed in creating lb pool: %s.'
                       ) % six.text_type(ex)
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self, 'POOL_CREATE',
-                          'ERROR', msg)
             return False, msg
         result['pool'] = pool.id
         res = self._wait_for_lb_ready(lb.id)
         if res is False:
             msg = _LE('Failed in creating pool (%s).') % pool.id
+            del result['vip_address']
+            _cleanup(msg, **result)
+            return res, msg
+
+        if not hm:
+            return True, result
+
+        if not hm:
+            return True, result
+
+        # Create health monitor
+        try:
+            health_monitor = self.nc().healthmonitor_create(
+                hm['type'], hm['delay'], hm['timeout'], hm['max_retries'],
+                pool.id, hm['admin_state_up'], hm['http_method'],
+                hm['url_path'], hm['expected_codes'])
+        except exception.InternalError as ex:
+            msg = _LE('Failed in creating lb health monitor: %s.'
+                      ) % six.text_type(ex)
+            LOG.exception(msg)
+            return False, msg
+        result['healthmonitor'] = health_monitor.id
+        res = self._wait_for_lb_ready(lb.id)
+        if res is False:
+            msg = _LE('Failed in creating health monitor (%s).'
+                      ) % health_monitor.id
+            del result['vip_address']
             _cleanup(msg, **result)
             return res, msg
 
@@ -179,8 +198,6 @@ class LoadBalancerDriver(base.DriverBase):
                 msg = _LE('Failed in deleting healthmonitor: %s.'
                           ) % six.text_type(ex)
                 LOG.exception(msg)
-                event.warning(oslo_context.get_current(), self,
-                              'HEALTHMONITOR_DELETE', 'ERROR', msg)
                 return False, msg
             res = self._wait_for_lb_ready(lb_id)
             if res is False:
@@ -196,8 +213,6 @@ class LoadBalancerDriver(base.DriverBase):
                 msg = _LE('Failed in deleting lb pool: %s.'
                           ) % six.text_type(ex)
                 LOG.exception(msg)
-                event.warning(oslo_context.get_current(), self, 'POOL_DELETE',
-                              'ERROR', msg)
                 return False, msg
             res = self._wait_for_lb_ready(lb_id)
             if res is False:
@@ -212,8 +227,6 @@ class LoadBalancerDriver(base.DriverBase):
                 msg = _LE('Failed in deleting listener: %s.'
                           ) % six.text_type(ex)
                 LOG.exception(msg)
-                event.warning(oslo_context.get_current(), self,
-                              'LISTENER_DELETE', 'ERROR', msg)
                 return False, msg
             res = self._wait_for_lb_ready(lb_id)
             if res is False:
@@ -238,12 +251,6 @@ class LoadBalancerDriver(base.DriverBase):
         :param subnet: The subnet to be used by the new LB member.
         :returns: The ID of the new LB member or None if errors occurred.
         """
-        addresses = self._get_node_address(node, version=4)
-        if not addresses:
-            LOG.error(_LE('Node (%(n)s) does not have valid IPv4 address.'),
-                      {'n': node.id})
-            return None
-
         try:
             subnet_obj = self.nc().subnet_get(subnet)
             net_id = subnet_obj.network_id
@@ -253,26 +260,36 @@ class LoadBalancerDriver(base.DriverBase):
             msg = _LE('Failed in getting %(resource)s: %(msg)s.'
                       ) % {'resource': resource, 'msg': six.text_type(ex)}
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self,
-                          resource.upper()+'_GET', 'ERROR', msg)
             return None
         net_name = net.name
 
+        node_detail = node.get_details(oslo_context.get_current())
+        addresses = node_detail.get('addresses')
         if net_name not in addresses:
-            LOG.error(_LE('Node is not in subnet %(subnet)s'),
-                      {'subnet': subnet})
+            msg = _LE('Node is not in subnet %(subnet)s')
+            LOG.error(msg, {'subnet': subnet})
             return None
 
-        address = addresses[net_name]
+        # Use the first IP address if more than one are found in target network
+        address = addresses[net_name][0]['addr']
         try:
+            # FIXME(Yanyan Hu): Currently, Neutron lbaasv2 service can not
+            # handle concurrent lb member operations well: new member creation
+            # deletion request will directly fail rather than being lined up
+            # when another operation is still in progress. In this workaround,
+            # loadbalancer status will be checked before creating lb member
+            # request is sent out. If loadbalancer keeps unready till waiting
+            # timeout, exception will be raised to fail member_add.
+            res = self._wait_for_lb_ready(lb_id)
+            if not res:
+                msg = _LE('Loadbalancer %s is not ready.') % lb_id
+                raise exception.Error(msg)
             member = self.nc().pool_member_create(pool_id, address, port,
                                                   subnet_obj.id)
-        except exception.InternalError as ex:
+        except (exception.InternalError, exception.Error) as ex:
             msg = _LE('Failed in creating lb pool member: %s.'
                       ) % six.text_type(ex)
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self,
-                          'POOL_MEMBER_CREATE', 'ERROR', msg)
             return None
         res = self._wait_for_lb_ready(lb_id)
         if res is False:
@@ -290,14 +307,23 @@ class LoadBalancerDriver(base.DriverBase):
         :returns: True if the operation succeeded or False if errors occurred.
         """
         try:
+            # FIXME(Yanyan Hu): Currently, Neutron lbaasv2 service can not
+            # handle concurrent lb member operations well: new member creation
+            # deletion request will directly fail rather than being lined up
+            # when another operation is still in progress. In this workaround,
+            # loadbalancer status will be checked before deleting lb member
+            # request is sent out. If loadbalancer keeps unready till waiting
+            # timeout, exception will be raised to fail member_remove.
+            res = self._wait_for_lb_ready(lb_id)
+            if not res:
+                msg = _LE('Loadbalancer %s is not ready.') % lb_id
+                raise exception.Error(msg)
             self.nc().pool_member_delete(pool_id, member_id)
-        except exception.InternalError as ex:
+        except (exception.InternalError, exception.Error) as ex:
             msg = _LE('Failed in removing member %(m)s from pool %(p)s: '
                       '%(ex)s') % {'m': member_id, 'p': pool_id,
                                    'ex': six.text_type(ex)}
             LOG.exception(msg)
-            event.warning(oslo_context.get_current(), self,
-                          'POOL_MEMBER_DELETE', 'ERROR', msg)
             return None
         res = self._wait_for_lb_ready(lb_id)
         if res is False:
@@ -305,18 +331,3 @@ class LoadBalancerDriver(base.DriverBase):
             return None
 
         return True
-
-    def _get_node_address(self, node, version=4):
-        """Get IP address of node with specific version"""
-
-        node_detail = node.get_details(oslo_context.get_current())
-        node_addresses = node_detail.get('addresses', {})
-
-        address = {}
-
-        for network in node_addresses:
-            for addr in node_addresses[network]:
-                if addr['version'] == version:
-                    address[network] = addr['addr']
-
-        return address
