@@ -10,13 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo_log import log
-
 from senlin.drivers import base
 from senlin.drivers.openstack import sdk
 
-LOG = log.getLogger(__name__)
-
 
 class NeutronClient(base.DriverBase):
     '''Neutron V2 driver.'''
@@ -25,22 +21,32 @@ class NeutronClient(base.DriverBase):
         super(NeutronClient, self).__init__(params)
         self.conn = sdk.create_connection(params)
 
-    def network_get(self, name_or_id):
-        network = self.conn.network.find_network(name_or_id)
+    @sdk.translate_exception
+    def network_get(self, name_or_id, ignore_missing=False):
+        network = self.conn.network.find_network(name_or_id, ignore_missing)
         return network
 
-    def subnet_get(self, name_or_id):
-        subnet = self.conn.network.find_subnet(name_or_id)
+    @sdk.translate_exception
+    def port_find(self, name_or_id, ignore_missing=False):
+        port = self.conn.network.find_port(name_or_id, ignore_missing)
+        return port
+
+    @sdk.translate_exception
+    def security_group_find(self, name_or_id, ignore_missing=False):
+        sg = self.conn.network.find_security_group(name_or_id, ignore_missing)
+        return sg
+
+    @sdk.translate_exception
+    def subnet_get(self, name_or_id, ignore_missing=False):
+        subnet = self.conn.network.find_subnet(name_or_id, ignore_missing)
         return subnet
 
-    def loadbalancer_get(self, name_or_id):
-        lb = self.conn.network.find_load_balancer(name_or_id)
+    @sdk.translate_exception
+    def loadbalancer_get(self, name_or_id, ignore_missing=False):
+        lb = self.conn.network.find_load_balancer(name_or_id, ignore_missing)
         return lb
 
-    def loadbalancer_list(self):
-        lbs = [lb for lb in self.conn.network.load_balancers()]
-        return lbs
-
+    @sdk.translate_exception
     def loadbalancer_create(self, vip_subnet_id, vip_address=None,
                             admin_state_up=True, name=None, description=None):
 
@@ -59,19 +65,13 @@ class NeutronClient(base.DriverBase):
         res = self.conn.network.create_load_balancer(**kwargs)
         return res
 
+    @sdk.translate_exception
     def loadbalancer_delete(self, lb_id, ignore_missing=True):
         self.conn.network.delete_load_balancer(
             lb_id, ignore_missing=ignore_missing)
         return
 
-    def listener_get(self, name_or_id):
-        listener = self.conn.network.find_listener(name_or_id)
-        return listener
-
-    def listener_list(self):
-        listeners = [i for i in self.conn.network.listeners()]
-        return listeners
-
+    @sdk.translate_exception
     def listener_create(self, loadbalancer_id, protocol, protocol_port,
                         connection_limit=None,
                         admin_state_up=True, name=None, description=None):
@@ -93,19 +93,13 @@ class NeutronClient(base.DriverBase):
         res = self.conn.network.create_listener(**kwargs)
         return res
 
+    @sdk.translate_exception
     def listener_delete(self, listener_id, ignore_missing=True):
         self.conn.network.delete_listener(listener_id,
                                           ignore_missing=ignore_missing)
         return
 
-    def pool_get(self, name_or_id):
-        pool = self.conn.network.find_pool(name_or_id)
-        return pool
-
-    def pool_list(self):
-        pools = [p for p in self.conn.network.pools()]
-        return pools
-
+    @sdk.translate_exception
     def pool_create(self, lb_algorithm, listener_id, protocol,
                     admin_state_up=True, name=None, description=None):
 
@@ -124,25 +118,17 @@ class NeutronClient(base.DriverBase):
         res = self.conn.network.create_pool(**kwargs)
         return res
 
+    @sdk.translate_exception
     def pool_delete(self, pool_id, ignore_missing=True):
         self.conn.network.delete_pool(pool_id,
                                       ignore_missing=ignore_missing)
         return
 
-    def pool_member_get(self, pool_id, name_or_id):
-        member = self.conn.network.find_pool_member(name_or_id,
-                                                    pool_id)
-        return member
-
-    def pool_member_list(self, pool_id):
-        members = [m for m in self.conn.network.pool_members(pool_id)]
-        return members
-
+    @sdk.translate_exception
     def pool_member_create(self, pool_id, address, protocol_port, subnet_id,
                            weight=None, admin_state_up=True):
 
         kwargs = {
-            'pool_id': pool_id,
             'address': address,
             'protocol_port': protocol_port,
             'admin_state_up': admin_state_up,
@@ -152,22 +138,16 @@ class NeutronClient(base.DriverBase):
         if weight is not None:
             kwargs['weight'] = weight
 
-        res = self.conn.network.create_pool_member(**kwargs)
+        res = self.conn.network.create_pool_member(pool_id, **kwargs)
         return res
 
+    @sdk.translate_exception
     def pool_member_delete(self, pool_id, member_id, ignore_missing=True):
         self.conn.network.delete_pool_member(
             member_id, pool_id, ignore_missing=ignore_missing)
         return
 
-    def healthmonitor_get(self, name_or_id):
-        hm = self.conn.network.find_health_monitor(name_or_id)
-        return hm
-
-    def healthmonitor_list(self):
-        hms = [hm for hm in self.conn.network.health_monitors()]
-        return hms
-
+    @sdk.translate_exception
     def healthmonitor_create(self, hm_type, delay, timeout, max_retries,
                              pool_id, admin_state_up=True,
                              http_method=None, url_path=None,
@@ -193,7 +173,46 @@ class NeutronClient(base.DriverBase):
         res = self.conn.network.create_health_monitor(**kwargs)
         return res
 
+    @sdk.translate_exception
     def healthmonitor_delete(self, hm_id, ignore_missing=True):
         self.conn.network.delete_health_monitor(
             hm_id, ignore_missing=ignore_missing)
         return
+
+    @sdk.translate_exception
+    def port_create(self, **attr):
+        res = self.conn.network.create_port(**attr)
+        return res
+
+    @sdk.translate_exception
+    def port_delete(self, port, ignore_missing=True):
+        res = self.conn.network.delete_port(
+            port=port, ignore_missing=ignore_missing)
+        return res
+
+    @sdk.translate_exception
+    def port_update(self, port, **attr):
+        res = self.conn.network.update_port(port, **attr)
+        return res
+
+    @sdk.translate_exception
+    def floatingip_find(self, name_or_id, ignore_missing=False):
+        res = self.conn.network.find_ip(
+            name_or_id, ignore_missing=ignore_missing)
+        return res
+
+    @sdk.translate_exception
+    def floatingip_create(self, **attr):
+        res = self.conn.network.create_ip(**attr)
+        return res
+
+    @sdk.translate_exception
+    def floatingip_delete(self, floating_ip, ignore_missing=True):
+        res = self.conn.network.delete_ip(
+            floating_ip, ignore_missing=ignore_missing)
+        return res
+
+    @sdk.translate_exception
+    def floatingip_update(self, floating_ip, **attr):
+        res = self.conn.network.update_ip(floating_ip, **attr)
+        return res
