@@ -17,26 +17,20 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
 
+from senlin.common import consts
 from senlin.common import context as req_context
 from senlin.common import exception
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
-from senlin.db import api as db_api
-from senlin.engine import cluster_policy as cp_mod
+from senlin.common.i18n import _, _LE
+from senlin.common import utils
 from senlin.engine import event as EVENT
+from senlin.objects import action as ao
+from senlin.objects import cluster_policy as cpo
+from senlin.objects import dependency as dobj
 from senlin.policies import base as policy_mod
 
 wallclock = time.time
 LOG = logging.getLogger(__name__)
 
-# Action causes
-CAUSES = (
-    CAUSE_RPC, CAUSE_DERIVED,
-) = (
-    'RPC Request',
-    'Derived Action',
-)
-
 
 class Action(object):
     '''An action can be performed on a cluster or a node of a cluster.'''
@@ -70,7 +64,7 @@ class Action(object):
         'CANCEL', 'SUSPEND', 'RESUME',
     )
 
-    def __new__(cls, target, action, context=None, **kwargs):
+    def __new__(cls, target, action, context, **kwargs):
         if (cls != Action):
             return super(Action, cls).__new__(cls)
 
@@ -87,27 +81,17 @@ class Action(object):
 
         return super(Action, cls).__new__(ActionClass)
 
-    def __init__(self, target, action, context=None, **kwargs):
+    def __init__(self, target, action, context, **kwargs):
         # context will be persisted into database so that any worker thread
         # can pick the action up and execute it on behalf of the initiator
 
         self.id = kwargs.get('id', None)
         self.name = kwargs.get('name', '')
 
-        # TODO(Yanyan Hu): Replace context with DB session
-        if not context:
-            params = {
-                'user': kwargs.get('user'),
-                'project': kwargs.get('project'),
-                'domain': kwargs.get('domain'),
-                'is_admin': False
-            }
-            self.context = req_context.RequestContext.from_dict(params)
-        else:
-            self.context = context
-
-        # TODO(Qiming): make description a db column
-        self.description = kwargs.get('description', '')
+        self.context = context
+        self.user = context.user
+        self.project = context.project
+        self.domain = context.domain
 
         self.action = action
         self.target = target
@@ -120,7 +104,7 @@ class Action(object):
         self.owner = kwargs.get('owner', None)
 
         # An action may need to be executed repeatitively, interval is the
-        # time in seconds between two consequtive execution.
+        # time in seconds between two consecutive execution.
         # A value of -1 indicates that this action is only to be executed once
         self.interval = kwargs.get('interval', -1)
 
@@ -145,20 +129,19 @@ class Action(object):
         self.inputs = kwargs.get('inputs', {})
         self.outputs = kwargs.get('outputs', {})
 
-        # Dependency with other actions
-        self.depends_on = kwargs.get('depends_on', [])
-        self.depended_by = kwargs.get('depended_by', [])
-
-        self.created_time = kwargs.get('created_time', None)
-        self.updated_time = kwargs.get('updated_time', None)
-        self.deleted_time = kwargs.get('deleted_time', None)
+        self.created_at = kwargs.get('created_at', None)
+        self.updated_at = kwargs.get('updated_at', None)
 
         self.data = kwargs.get('data', {})
 
     def store(self, context):
-        '''Store the action record into database table.'''
+        """Store the action record into database table.
 
-        timestamp = timeutils.utcnow()
+        :param context: An instance of the request context.
+        :return: The ID of the stored object.
+        """
+
+        timestamp = timeutils.utcnow(True)
 
         values = {
             'name': self.name,
@@ -175,111 +158,127 @@ class Action(object):
             'status_reason': self.status_reason,
             'inputs': self.inputs,
             'outputs': self.outputs,
-            'depends_on': self.depends_on,
-            'depended_by': self.depended_by,
-            'created_time': self.created_time,
-            'updated_time': self.updated_time,
-            'deleted_time': self.deleted_time,
+            'created_at': self.created_at,
+            'updated_at': self.updated_at,
             'data': self.data,
+            'user': self.user,
+            'project': self.project,
+            'domain': self.domain,
         }
 
         if self.id:
-            self.updated_time = timestamp
-            values['updated_time'] = timestamp
-            db_api.action_update(context, self.id, values)
+            self.updated_at = timestamp
+            values['updated_at'] = timestamp
+            ao.Action.update(context, self.id, values)
         else:
-            self.created_time = timestamp
-            values['created_time'] = timestamp
-            action = db_api.action_create(context, values)
+            self.created_at = timestamp
+            values['created_at'] = timestamp
+            action = ao.Action.create(context, values)
             self.id = action.id
 
         return self.id
 
     @classmethod
-    def _from_db_record(cls, record):
-        '''Construct a action object from database record.
+    def _from_object(cls, obj):
+        """Construct an action from database object.
 
-        :param context: the context used for DB operations;
-        :param record: a DB action object that contains all fields.
-        '''
-        context = req_context.RequestContext.from_dict(record.context)
+        :param obj: a DB action object that contains all fields.
+        :return: An `Action` object deserialized from the DB action object.
+        """
+        context = req_context.RequestContext.from_dict(obj.context)
         kwargs = {
-            'id': record.id,
-            'name': record.name,
-            'cause': record.cause,
-            'owner': record.owner,
-            'interval': record.interval,
-            'start_time': record.start_time,
-            'end_time': record.end_time,
-            'timeout': record.timeout,
-            'status': record.status,
-            'status_reason': record.status_reason,
-            'inputs': record.inputs,
-            'outputs': record.outputs,
-            'depends_on': record.depends_on,
-            'depended_by': record.depended_by,
-            'created_time': record.created_time,
-            'updated_time': record.updated_time,
-            'deleted_time': record.deleted_time,
-            'data': record.data,
+            'id': obj.id,
+            'name': obj.name,
+            'cause': obj.cause,
+            'owner': obj.owner,
+            'interval': obj.interval,
+            'start_time': obj.start_time,
+            'end_time': obj.end_time,
+            'timeout': obj.timeout,
+            'status': obj.status,
+            'status_reason': obj.status_reason,
+            'inputs': obj.inputs or {},
+            'outputs': obj.outputs or {},
+            'created_at': obj.created_at,
+            'updated_at': obj.updated_at,
+            'data': obj.data,
         }
 
-        return cls(record.target, record.action, context=context, **kwargs)
+        return cls(obj.target, obj.action, context, **kwargs)
 
     @classmethod
-    def load(cls, context, action_id=None, db_action=None, show_deleted=False):
-        '''Retrieve an action from database.'''
+    def load(cls, context, action_id=None, db_action=None, project_safe=True):
+        """Retrieve an action from database.
+
+        :param context: Instance of request context.
+        :param action_id: An UUID for the action to deserialize.
+        :param db_action: An action object for the action to deserialize.
+        :return: A `Action` object instance.
+        """
         if db_action is None:
-            db_action = db_api.action_get(context, action_id,
-                                          show_deleted=show_deleted)
-
+            db_action = ao.Action.get(context, action_id,
+                                      project_safe=project_safe)
             if db_action is None:
-                raise exception.ActionNotFound(action=action_id)
+                raise exception.ResourceNotFound(type='action', id=action_id)
 
-        return cls._from_db_record(db_action)
+        return cls._from_object(db_action)
 
     @classmethod
-    def load_all(cls, context, filters=None, limit=None, marker=None,
-                 sort_keys=None, sort_dir=None, show_deleted=False):
-        '''Retrieve all actions of from database.'''
+    def create(cls, context, target, action, **kwargs):
+        """Create an action object.
 
-        records = db_api.action_get_all(context, filters=filters,
-                                        limit=limit, marker=marker,
-                                        sort_keys=sort_keys,
-                                        sort_dir=sort_dir,
-                                        show_deleted=show_deleted)
-
-        for record in records:
-            yield cls._from_db_record(record)
+        :param context: The requesting context.
+        :param target: The ID of the target cluster/node.
+        :param action: Name of the action.
+        :param dict kwargs: Other keyword arguments for the action.
+        :return: ID of the action created.
+        """
+        params = {
+            'user': context.user,
+            'project': context.project,
+            'domain': context.domain,
+            'is_admin': context.is_admin,
+            'request_id': context.request_id,
+            'trusts': context.trusts,
+        }
+        ctx = req_context.RequestContext.from_dict(params)
+        obj = cls(target, action, ctx, **kwargs)
+        return obj.store(context)
 
     @classmethod
-    def delete(cls, context, action_id, force=False):
-        db_api.action_delete(context, action_id, force)
+    def delete(cls, context, action_id):
+        """Delete an action from database.
+
+        :param context: An instance of the request context.
+        :param action_id: The UUID of the target action to be deleted.
+        :return: Nothing.
+        """
+        ao.Action.delete(context, action_id)
 
     def signal(self, cmd):
-        '''Send a signal to the action.'''
+        """Send a signal to the action.
+
+        :param cmd: One of the command word defined in self.COMMANDS.
+        :returns: None
+        """
         if cmd not in self.COMMANDS:
             return
 
         if cmd == self.SIG_CANCEL:
-            expected_statuses = (self.INIT, self.WAITING, self.READY,
-                                 self.RUNNING)
+            expected = (self.INIT, self.WAITING, self.READY, self.RUNNING)
         elif cmd == self.SIG_SUSPEND:
-            expected_statuses = (self.RUNNING)
-        else:     # SIG_RESUME
-            expected_statuses = (self.SUSPENDED)
+            expected = (self.RUNNING)
+        else:  # SIG_RESUME
+            expected = (self.SUSPENDED)
 
-        if self.status not in expected_statuses:
-            reason = _("Action (%(action)s) is in unexpected status "
-                       "(%(actual)s) while expected status should be one of "
-                       "(%(expected)s).") % dict(action=self.id,
-                                                 expected=expected_statuses,
-                                                 actual=self.status)
-            EVENT.error(self.context, self, cmd, status_reason=reason)
+        if self.status not in expected:
+            LOG.error(_LE("Action (%(id)s) is in status (%(actual)s) while "
+                          "expected status should be one of (%(expected)s)."),
+                      dict(id=self.id[:8], expected=expected,
+                           actual=self.status))
             return
 
-        # TODO(Yanyan Hu): use DB session here
-        db_api.action_signal(self.context, self.id, cmd)
+        ao.Action.signal(self.context, self.id, cmd)
 
     def execute(self, **kwargs):
         '''Execute the action.
@@ -289,7 +288,7 @@ class Action(object):
         :param kwargs: additional parameters that may override the default
                        properties stored in the action record.
         '''
-        return NotImplemented
+        raise NotImplementedError
 
     def set_status(self, result, reason=None):
         """Set action status based on return value from execute."""
@@ -298,42 +297,43 @@ class Action(object):
 
         if result == self.RES_OK:
             status = self.SUCCEEDED
-            db_api.action_mark_succeeded(self.context, self.id, timestamp)
+            ao.Action.mark_succeeded(self.context, self.id, timestamp)
 
         elif result == self.RES_ERROR:
             status = self.FAILED
-            db_api.action_mark_failed(self.context, self.id, timestamp,
-                                      reason=reason or 'ERROR')
+            ao.Action.mark_failed(self.context, self.id, timestamp,
+                                  reason or 'ERROR')
 
         elif result == self.RES_TIMEOUT:
             status = self.FAILED
-            db_api.action_mark_failed(self.context, self.id, timestamp,
-                                      reason=reason or 'TIMEOUT')
+            ao.Action.mark_failed(self.context, self.id, timestamp,
+                                  reason or 'TIMEOUT')
 
         elif result == self.RES_CANCEL:
             status = self.CANCELLED
-            db_api.action_mark_cancelled(self.context, self.id, timestamp)
+            ao.Action.mark_cancelled(self.context, self.id, timestamp)
 
         else:  # result == self.RES_RETRY:
             status = self.READY
             # Action failed at the moment, but can be retried
             # We abandon it and then notify other dispatchers to execute it
-            db_api.action_abandon(self.context, self.id)
+            ao.Action.abandon(self.context, self.id)
 
         if status == self.SUCCEEDED:
-            EVENT.info(self.context, self, self.action, status, reason)
+            EVENT.info(self, consts.PHASE_END, reason or 'SUCCEEDED')
         elif status == self.READY:
-            EVENT.warning(self.context, self, self.action, status, reason)
+            EVENT.warning(self, consts.PHASE_ERROR, reason or 'RETRY')
         else:
-            EVENT.error(self.context, self, self.action, status, reason)
+            EVENT.error(self, consts.PHASE_ERROR, reason or 'ERROR')
 
         self.status = status
         self.status_reason = reason
 
     def get_status(self):
-        action = db_api.action_get(self.context, self.id, refresh=True)
-        self.status = action.status
-        return action.status
+        timestamp = wallclock()
+        status = ao.Action.check_status(self.context, self.id, timestamp)
+        self.status = status
+        return status
 
     def is_timeout(self):
         time_lapse = wallclock() - self.start_time
@@ -342,10 +342,10 @@ class Action(object):
     def _check_signal(self):
         # Check timeout first, if true, return timeout message
         if self.timeout is not None and self.is_timeout():
-            EVENT.debug(self.context, self, self.action, 'TIMEOUT')
+            EVENT.debug(self, consts.PHASE_ERROR, 'TIMEOUT')
             return self.RES_TIMEOUT
 
-        result = db_api.action_signal_query(self.context, self.id)
+        result = ao.Action.signal_query(self.context, self.id)
         return result
 
     def is_cancelled(self):
@@ -357,40 +357,22 @@ class Action(object):
     def is_resumed(self):
         return self._check_signal() == self.SIG_RESUME
 
-    def _check_result(self, level, name):
+    def _check_result(self, name):
         """Check policy check status and generate event.
 
-        :param level: Enforcement level of the policy (when bound to cluster).
         :param name: Name of policy checked
         :return: True if the policy checking can be continued, or False if the
                  policy checking should be aborted.
         """
-        # Abort policy checking if failures found
-        status = 'CHECK ERROR'
-        reason = _("Failed policy '%(name)s': %(reason)s."
-                   ) % {'name': name, 'reason': self.data['reason']}
+        reason = self.data['reason']
         if self.data['status'] == policy_mod.CHECK_OK:
-            method = EVENT.debug
-            status = 'CHECK OK'
-            reason = self.data['reason']
-        elif level >= policy_mod.MUST:
-            method = EVENT.critical
-        elif level >= policy_mod.SHOULD:
-            method = EVENT.error
-        elif level >= policy_mod.WOULD:
-            method = EVENT.warning
-        elif level >= policy_mod.MIGHT:
-            method = EVENT.info
-        else:
-            method = EVENT.debug
-
-        method(self.context, self, self.action, status, reason)
-
-        if ((self.data['status'] == policy_mod.CHECK_OK) or
-                (level < policy_mod.SHOULD)):
+            EVENT.debug(self, 'check', reason)
             return True
-        else:
-            return False
+
+        reason = _("Failed policy '%(name)s': %(reason)s"
+                   ) % {'name': name, 'reason': reason}
+        EVENT.error(self, consts.PHASE_ERROR, reason)
+        return False
 
     def policy_check(self, cluster_id, target):
         """Check all policies attached to cluster and give result.
@@ -404,10 +386,9 @@ class Action(object):
         if target not in ['BEFORE', 'AFTER']:
             return
 
-        # TODO(Anyone): This could use the cluster's runtime data
-        bindings = cp_mod.ClusterPolicy.load_all(self.context, cluster_id,
-                                                 sort_keys=['priority'],
-                                                 filters={'enabled': True})
+        bindings = cpo.ClusterPolicy.get_all(self.context, cluster_id,
+                                             sort='priority',
+                                             filters={'enabled': True})
         # default values
         self.data['status'] = policy_mod.CHECK_OK
         self.data['reason'] = _('Completed policy checking.')
@@ -418,9 +399,12 @@ class Action(object):
             # cluster, no matter that policy is only interested in the
             # "BEFORE" or "AFTER" or both.
             if target == 'AFTER':
-                pb.record_last_op(self.context)
+                ts = timeutils.utcnow(True)
+                pb.last_op = ts
+                cpo.ClusterPolicy.update(self.context, pb.cluster_id,
+                                         pb.policy_id, {'last_op': ts})
 
-            if (target, self.action) not in policy.TARGET:
+            if not policy.need_check(target, self):
                 continue
 
             if target == 'BEFORE':
@@ -428,85 +412,81 @@ class Action(object):
             else:  # target == 'AFTER'
                 method = getattr(policy, 'post_op', None)
 
-            if pb.cooldown_inprogress():
-                self.data['status'] = policy_mod.CHECK_ERROR
-                self.data['reason'] = _('Policy %(id)s cooldown is still '
-                                        'in progress.') % {'id': policy.id}
+            if getattr(policy, 'cooldown', None):
+                if pb.cooldown_inprogress(policy.cooldown):
+                    self.data['status'] = policy_mod.CHECK_ERROR
+                    self.data['reason'] = _('Policy %s cooldown is still '
+                                            'in progress.') % policy.id
                 return
 
             if method is not None:
                 method(cluster_id, self)
 
-            res = self._check_result(pb.level, policy.name)
+            res = self._check_result(policy.name)
             if res is False:
                 return
         return
 
     def to_dict(self):
+        if self.id:
+            dep_on = dobj.Dependency.get_depended(self.context, self.id)
+            dep_by = dobj.Dependency.get_dependents(self.context, self.id)
+        else:
+            dep_on = []
+            dep_by = []
         action_dict = {
             'id': self.id,
             'name': self.name,
             'action': self.action,
-            'context': self.context.to_dict(),
             'target': self.target,
             'cause': self.cause,
             'owner': self.owner,
             'interval': self.interval,
             'start_time': self.start_time,
             'end_time': self.end_time,
-            'interval': self.interval,
             'timeout': self.timeout,
             'status': self.status,
             'status_reason': self.status_reason,
             'inputs': self.inputs,
             'outputs': self.outputs,
-            'depends_on': self.depends_on,
-            'depended_by': self.depended_by,
-            'created_time': self.created_time,
-            'updated_time': self.updated_time,
-            'deleted_time': self.deleted_time,
+            'depends_on': dep_on,
+            'depended_by': dep_by,
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
             'data': self.data,
+            'user': self.user,
+            'project': self.project,
         }
         return action_dict
 
 
-# TODO(Yanyan Hu): Replace context parameter with session parameter
-def ActionProc(context, action_id, worker_id):
+def ActionProc(context, action_id):
     '''Action process.'''
 
     # Step 1: materialize the action object
-    action = Action.load(context, action_id=action_id)
+    action = Action.load(context, action_id=action_id, project_safe=False)
     if action is None:
         LOG.error(_LE('Action "%s" could not be found.'), action_id)
         return False
 
-    # Step 2: lock the action for execution
-    timestamp = wallclock()
-    res = db_api.action_acquire(action.context, action_id, worker_id,
-                                timestamp)
-    if res is None:
-        LOG.warning(_LE('Failed in locking action "%s".'), action_id)
-        return False
-
-    action.owner = res.owner
-    action.start_time = res.start_time
-    # TODO(Anyone): Remove context usage in event module
-    EVENT.info(action.context, action, action.action, 'START')
+    EVENT.info(action, consts.PHASE_START)
 
     reason = 'Action completed'
     success = True
     try:
-        # Step 3: execute the action
+        # Step 2: execute the action
         result, reason = action.execute()
+        if result == action.RES_RETRY:
+            success = False
     except Exception as ex:
         # We catch exception here to make sure the following logics are
         # executed.
         result = action.RES_ERROR
         reason = six.text_type(ex)
-        LOG.exception(_('Unexpected exception occurred during action '
-                        '%(action)s (%(id)s) execution: %(reason)s'),
+        LOG.exception(_LE('Unexpected exception occurred during action '
+                          '%(action)s (%(id)s) execution: %(reason)s'),
                       {'action': action.action, 'id': action.id,
-                       'reason': reason})
+                      'reason': reason})
         success = False
     finally:
         # NOTE: locks on action is eventually released here by status update
