@@ -11,12 +11,10 @@
 # under the License.
 
 import itertools
-import six
 
 from oslo_log import log as logging
 
-from senlin.common.i18n import _LI
-from senlin.common.i18n import _LW
+from senlin.common.i18n import _LI, _LW
 
 LOG = logging.getLogger(__name__)
 
@@ -83,7 +81,8 @@ class Registry(object):
         registry = self._registry
         if info is None:
             # delete this entry.
-            LOG.warn(_LW('Removing %(item)s from registry'), {'item': name})
+            msg = _LW("Removing %(item)s from registry")
+            LOG.warning(msg, {'item': name})
             registry.pop(name, None)
             return
 
@@ -95,10 +94,11 @@ class Registry(object):
                 'old': str(registry[name].plugin),
                 'new': str(info.plugin)
             }
-            LOG.warn(_LW('Changing %(name)s from %(old)s to %(new)s'), details)
+            LOG.warning(_LW('Changing %(name)s from %(old)s to %(new)s'),
+                        details)
         else:
-            LOG.info(_LI('Registering %(name)s -> %(value)s'), {
-                'name': name, 'value': str(info.plugin)})
+            msg = _LI('Registering %(name)s -> %(value)s')
+            LOG.info(msg, {'name': name, 'value': str(info.plugin)})
 
         info.user_provided = not self.is_global
         registry[name] = info
@@ -125,12 +125,19 @@ class Registry(object):
             giter = self.global_registry.iterable_by(name)
 
         matches = itertools.chain(self.iterable_by(name), giter)
-        infoes = sorted(matches)
-        return infoes[0].plugin if infoes else None
+        infos = sorted(matches)
+        return infos[0].plugin if infos else None
 
     def as_dict(self):
         return dict((k, v.plugin) for k, v in self._registry.items())
 
     def get_types(self):
         '''Return a list of valid plugin types.'''
-        return [{'name': name} for name in six.iterkeys(self._registry)]
+        return [
+            {
+                'name': name.split('-')[0] if '-' in name else name,
+                'version': name.split('-')[1] if '-' in name else '',
+                'support_status': pi.plugin.VERSIONS
+            }
+            for name, pi in self._registry.items()
+        ]
