@@ -12,10 +12,13 @@
 
 from oslo_config import cfg
 from oslo_log import log as logging
+import time
 
-from senlin.common.i18n import _LE
-from senlin.db import api as db_api
-from senlin.engine import scheduler
+from senlin.common.i18n import _, _LE, _LI
+from senlin.common import utils
+from senlin.objects import action as ao
+from senlin.objects import cluster_lock as cl_obj
+from senlin.objects import node_lock as nl_obj
 
 CONF = cfg.CONF
 
@@ -31,12 +34,13 @@ LOCK_SCOPES = (
 )
 
 
-def cluster_lock_acquire(cluster_id, action_id, scope=CLUSTER_SCOPE,
-                         forced=False):
+def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
+                         scope=CLUSTER_SCOPE, forced=False):
     """Try to lock the specified cluster.
 
     :param cluster_id: ID of the cluster to be locked.
     :param action_id: ID of the action which wants to lock the cluster.
+    :param engine: ID of the engine which wants to lock the cluster.
     :param scope: scope of lock, could be cluster wide lock, or node-wide
                   lock.
     :param forced: set to True to cancel current action that owns the lock,
@@ -46,24 +50,29 @@ def cluster_lock_acquire(cluster_id, action_id, scope=CLUSTER_SCOPE,
 
     # Step 1: try lock the cluster - if the returned owner_id is the
     #         action id, it was a success
-    owners = db_api.cluster_lock_acquire(cluster_id, action_id, scope)
+    owners = cl_obj.ClusterLock.acquire(cluster_id, action_id, scope)
     if action_id in owners:
         return True
 
-    # Step 2: retry using global configuration options
-    retries = cfg.CONF.lock_retry_times
-    retry_interval = cfg.CONF.lock_retry_interval
-
-    while retries > 0:
-        scheduler.sleep(retry_interval)
-        owners = db_api.cluster_lock_acquire(cluster_id, action_id, scope)
-        if action_id in owners:
-            return True
-        retries = retries - 1
-
-    # Step 3: Last resort is 'forced locking', only needed when retry failed
+    # Step 2: Last resort is 'forced locking', only needed when retry failed
     if forced:
-        owners = db_api.cluster_lock_steal(cluster_id, action_id)
+        owners = cl_obj.ClusterLock.steal(cluster_id, action_id)
+        return action_id in owners
+
+    # Step 3: check if the owner is a dead engine, if so, steal the lock.
+    # Will reach here only because scope == CLUSTER_SCOPE
+    action = ao.Action.get(context, owners[0])
+    if (action and action.owner and action.owner != engine and
+            utils.is_engine_dead(context, action.owner)):
+        LOG.info(_LI('The cluster %(c)s is locked by dead action %(a)s, '
+                     'try to steal the lock.'), {
+            'c': cluster_id,
+            'a': owners[0]
+        })
+        reason = _('Engine died when executing this action.')
+        owners = cl_obj.ClusterLock.steal(cluster_id, action_id)
+        # Mark the old action to failed.
+        ao.Action.mark_failed(context, action.id, time.time(), reason)
         return action_id in owners
 
     LOG.error(_LE('Cluster is already locked by action %(old)s, '
@@ -80,40 +89,47 @@ def cluster_lock_release(cluster_id, action_id, scope):
     :param action_id: ID of the action that attempts to release the node.
     :param scope: The scope of the lock to be released.
     """
-    return db_api.cluster_lock_release(cluster_id, action_id, scope)
+    return cl_obj.ClusterLock.release(cluster_id, action_id, scope)
 
 
-def node_lock_acquire(node_id, action_id, forced=False):
+def node_lock_acquire(context, node_id, action_id, engine=None,
+                      forced=False):
     """Try to lock the specified node.
 
+    :param context: the context used for DB operations;
     :param node_id: ID of the node to be locked.
     :param action_id: ID of the action that attempts to lock the node.
+    :param engine: ID of the engine that attempts to lock the node.
     :param forced: set to True to cancel current action that owns the lock,
                    if any.
     :returns: True if lock is acquired, or False otherwise.
     """
     # Step 1: try lock the node - if the returned owner_id is the
     #         action id, it was a success
-    owner = db_api.node_lock_acquire(node_id, action_id)
+    owner = nl_obj.NodeLock.acquire(node_id, action_id)
     if action_id == owner:
         return True
 
-    # Step 2: retry using global configuration options
-    retries = cfg.CONF.lock_retry_times
-    retry_interval = cfg.CONF.lock_retry_interval
-
-    while retries > 0:
-        scheduler.sleep(retry_interval)
-        owner = db_api.node_lock_acquire(node_id, action_id)
-        if action_id == owner:
-            return True
-        retries = retries - 1
-
-    # Step 3: Last resort is 'forced locking', only needed when retry failed
+    # Step 2: Last resort is 'forced locking', only needed when retry failed
     if forced:
-        owner = db_api.node_lock_steal(node_id, action_id)
+        owner = nl_obj.NodeLock.steal(node_id, action_id)
         return action_id == owner
 
+    # Step 3: Try to steal a lock if it's owner is a dead engine.
+    # if this node lock by dead engine
+    action = ao.Action.get(context, owner)
+    if (action and action.owner and action.owner != engine and
+            utils.is_engine_dead(context, action.owner)):
+        LOG.info(_LI('The node %(n)s is locked by dead action %(a)s, '
+                     'try to steal the lock.'), {
+            'n': node_id,
+            'a': owner
+        })
+        reason = _('Engine died when executing this action.')
+        nl_obj.NodeLock.steal(node_id, action_id)
+        ao.Action.mark_failed(context, action.id, time.time(), reason)
+        return True
+
     LOG.error(_LE('Node is already locked by action %(old)s, '
                   'action %(new)s failed grabbing the lock'),
               {'old': owner, 'new': action_id})
@@ -127,4 +143,4 @@ def node_lock_release(node_id, action_id):
     :param node_id: ID of the node to be released.
     :param action_id: ID of the action that attempts to release the node.
     """
-    return db_api.node_lock_release(node_id, action_id)
+    return nl_obj.NodeLock.release(node_id, action_id)
