@@ -10,38 +10,58 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+"""
+Policy for load-balancing among nodes in a cluster.
+
+NOTE: For full documentation about how the deletion policy works, check:
+http://docs.openstack.org/developer/senlin/developer/policies/
+load_balance_v1.html
+"""
+
 from oslo_context import context as oslo_context
 from oslo_log import log as logging
 
 from senlin.common import constraints
 from senlin.common import consts
-from senlin.common.i18n import _
-from senlin.common.i18n import _LW
+from senlin.common import exception as exc
+from senlin.common.i18n import _, _LW
+from senlin.common import scaleutils
 from senlin.common import schema
-from senlin.db import api as db_api
-from senlin.drivers import base as driver_base
 from senlin.engine import cluster_policy
-from senlin.engine import node as node_mod
+from senlin.engine import node as nm
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 LOG = logging.getLogger(__name__)
 
 
 class LoadBalancingPolicy(base.Policy):
-    '''Policy for load balancing among members of a cluster.
+    """Policy for load balancing among members of a cluster.
 
-    This policy is expected to be enforced after the member list of a cluster
-    is changed. We need to reload the load-balancer specified (or internally
-    created) when these actions are performed.
-    '''
-    VERSION = '1.0'
+    This policy is expected to be enforced before or after the membership of a
+    cluster is changed. We need to refresh the load-balancer associated with
+    the cluster (which could be created by the policy) when these actions are
+    performed.
+    """
+    VERSION = '1.1'
+    VERSIONS = {
+        '1.0': [
+            {'status': consts.SUPPORTED, 'since': '2016.04'}
+        ]
+    }
+
+    PRIORITY = 500
 
     TARGET = [
         ('AFTER', consts.CLUSTER_ADD_NODES),
-        ('AFTER', consts.CLUSTER_DEL_NODES),
         ('AFTER', consts.CLUSTER_SCALE_OUT),
-        ('AFTER', consts.CLUSTER_SCALE_IN),
         ('AFTER', consts.CLUSTER_RESIZE),
+        ('AFTER', consts.NODE_CREATE),
+        ('BEFORE', consts.CLUSTER_DEL_NODES),
+        ('BEFORE', consts.CLUSTER_SCALE_IN),
+        ('BEFORE', consts.CLUSTER_RESIZE),
+        ('BEFORE', consts.NODE_DELETE),
     ]
 
     PROFILE_TYPE = [
@@ -49,9 +69,9 @@ class LoadBalancingPolicy(base.Policy):
     ]
 
     KEYS = (
-        POOL, VIP,
+        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT
     ) = (
-        'pool', 'vip',
+        'pool', 'vip', 'health_monitor', 'lb_status_timeout'
     )
 
     _POOL_KEYS = (
@@ -74,6 +94,18 @@ class LoadBalancingPolicy(base.Policy):
         'ROUND_ROBIN', 'LEAST_CONNECTIONS', 'SOURCE_IP',
     )
 
+    HEALTH_MONITOR_TYPES = (
+        PING, TCP, HTTP, HTTPS,
+    ) = (
+        'PING', 'TCP', 'HTTP', 'HTTPS',
+    )
+
+    HTTP_METHODS = (
+        GET, POST, PUT, DELETE,
+    ) = (
+        'GET', 'POST', 'PUT', 'DELETE',
+    )
+
     _VIP_KEYS = (
         VIP_SUBNET, VIP_ADDRESS, VIP_CONNECTION_LIMIT, VIP_PROTOCOL,
         VIP_PROTOCOL_PORT, VIP_ADMIN_STATE_UP,
@@ -82,6 +114,14 @@ class LoadBalancingPolicy(base.Policy):
         'protocol_port', 'admin_state_up',
     )
 
+    HEALTH_MONITOR_KEYS = (
+        HM_TYPE, HM_DELAY, HM_TIMEOUT, HM_MAX_RETRIES, HM_ADMIN_STATE_UP,
+        HM_HTTP_METHOD, HM_URL_PATH, HM_EXPECTED_CODES,
+    ) = (
+        'type', 'delay', 'timeout', 'max_retries', 'admin_state_up',
+        'http_method', 'url_path', 'expected_codes',
+    )
+
     _SESSION_PERSISTENCE_KEYS = (
         PERSISTENCE_TYPE, COOKIE_NAME,
     ) = (
@@ -126,7 +166,7 @@ class LoadBalancingPolicy(base.Policy):
                     default=True,
                 ),
                 POOL_SESSION_PERSISTENCE: schema.Map(
-                    _('Session pesistence configuration.'),
+                    _('Session persistence configuration.'),
                     schema={
                         PERSISTENCE_TYPE: schema.String(
                             _('Type of session persistence implementation.'),
@@ -176,6 +216,58 @@ class LoadBalancingPolicy(base.Policy):
                 ),
             },
         ),
+        HEALTH_MONITOR: schema.Map(
+            _('Health monitor for loadbalancer.'),
+            schema={
+                HM_TYPE: schema.String(
+                    _('The type of probe sent by the loadbalancer to verify '
+                      'the member state.'),
+                    constraints=[
+                        constraints.AllowedValues(HEALTH_MONITOR_TYPES),
+                    ],
+                    default=PING,
+                ),
+                HM_DELAY: schema.Integer(
+                    _('The amount of time in milliseconds between sending '
+                      'probes to members.'),
+                    default=10,
+                ),
+                HM_TIMEOUT: schema.Integer(
+                    _('The maximum time in milliseconds that a monitor waits '
+                      'to connect before it times out.'),
+                    default=5,
+                ),
+                HM_MAX_RETRIES: schema.Integer(
+                    _('The number of allowed connection failures before '
+                      'changing the status of the member to INACTIVE.'),
+                    default=3,
+                ),
+                HM_ADMIN_STATE_UP: schema.Boolean(
+                    _('Administrative state of the health monitor.'),
+                    default=True,
+                ),
+                HM_HTTP_METHOD: schema.String(
+                    _('The HTTP method that the monitor uses for requests.'),
+                    constraints=[
+                        constraints.AllowedValues(HTTP_METHODS),
+                    ],
+                ),
+                HM_URL_PATH: schema.String(
+                    _('The HTTP path of the request sent by the monitor to '
+                      'test the health of a member.'),
+                ),
+                HM_EXPECTED_CODES: schema.String(
+                    _('Expected HTTP codes for a passing HTTP(S) monitor.'),
+                ),
+            },
+        ),
+        LB_STATUS_TIMEOUT: schema.Integer(
+            _('Time in second to wait for loadbalancer to be ready'
+              '(provisioning_status is ACTIVE and operating_status is '
+              'ONLINE) before and after senlin requests lbaas V2 service '
+              'for lb operations. '),
+            default=600,
+        )
     }
 
     def __init__(self, name, spec, **kwargs):
@@ -183,19 +275,41 @@ class LoadBalancingPolicy(base.Policy):
 
         self.pool_spec = self.properties.get(self.POOL, {})
         self.vip_spec = self.properties.get(self.VIP, {})
-        self.validate()
+        self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
+        self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
         self.lb = None
 
-    def validate(self):
-        super(LoadBalancingPolicy, self).validate()
+    def validate(self, context, validate_props=False):
+        super(LoadBalancingPolicy, self).validate(context, validate_props)
 
-        # validate subnet's exists
-        # subnet = self.nc.subnet_get(vip[self.VIP_SUBNET])
+        if not validate_props:
+            return True
 
-    def attach(self, cluster):
+        nc = self.network(context.user, context.project)
+
+        # validate pool subnet
+        name_or_id = self.pool_spec.get(self.POOL_SUBNET)
+        try:
+            nc.subnet_get(name_or_id)
+        except exc.InternalError:
+            msg = _("The specified %(key)s '%(value)s' could not be found."
+                    ) % {'key': self.POOL_SUBNET, 'value': name_or_id}
+            raise exc.InvalidSpec(message=msg)
+
+        # validate VIP subnet
+        name_or_id = self.vip_spec.get(self.VIP_SUBNET)
+        try:
+            nc.subnet_get(name_or_id)
+        except exc.InternalError:
+            msg = _("The specified %(key)s '%(value)s' could not be found."
+                    ) % {'key': self.VIP_SUBNET, 'value': name_or_id}
+            raise exc.InvalidSpec(message=msg)
+
+    def attach(self, cluster, enabled=True):
         """Routine to be invoked when policy is to be attached to a cluster.
 
-        :param cluster: The target cluster to be attached to;
+        :param cluster: The cluster to which the policy is being attached to.
+        :param enabled: The attached cluster policy is enabled or disabled.
         :returns: When the operation was successful, returns a tuple (True,
                   message); otherwise, return a tuple (False, error).
         """
@@ -203,13 +317,17 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, data
 
-        nodes = node_mod.Node.load_all(oslo_context.get_current(),
-                                       cluster_id=cluster.id)
+        nodes = nm.Node.load_all(oslo_context.get_current(),
+                                 cluster_id=cluster.id)
 
-        params = self._build_conn_params(cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        lb_driver = self.lbaas(cluster.user, cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
 
-        res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec)
+        # TODO(Anyone): Check if existing nodes has conflicts regarding the
+        # subnets. Each VM addresses detail has a key named to the network
+        # which can be used for validation.
+        res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
+                                        self.hm_spec)
         if res is False:
             return False, data
 
@@ -222,15 +340,18 @@ class LoadBalancingPolicy(base.Policy):
             if member_id is None:
                 # When failed in adding member, remove all lb resources that
                 # were created and return the failure reason.
-                # TODO(Yanyan Hu): Maybe we should tolerate member adding
-                # failure and allow policy attaching to succeed without
-                # all nodes being added into lb pool?
+                # TODO(anyone): May need to "roll-back" changes caused by any
+                # successful member_add() calls.
                 lb_driver.lb_delete(**data)
                 return False, 'Failed in adding node into lb pool'
 
             node.data.update({'lb_member': member_id})
             node.store(oslo_context.get_current())
 
+        cluster_data_lb = cluster.data.get('loadbalancers', {})
+        cluster_data_lb[self.id] = {'vip_address': data.pop('vip_address')}
+        cluster.data['loadbalancers'] = cluster_data_lb
+
         policy_data = self._build_policy_data(data)
 
         return True, policy_data
@@ -245,8 +366,8 @@ class LoadBalancingPolicy(base.Policy):
             contains a error message.
         """
         reason = _('LB resources deletion succeeded.')
-        params = self._build_conn_params(cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        lb_driver = self.lbaas(cluster.user, cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
 
         cp = cluster_policy.ClusterPolicy.load(oslo_context.get_current(),
                                                cluster.id, self.id)
@@ -259,17 +380,70 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, reason
 
-        nodes = node_mod.Node.load_all(oslo_context.get_current(),
-                                       cluster_id=cluster.id)
+        nodes = nm.Node.load_all(oslo_context.get_current(),
+                                 cluster_id=cluster.id, project_safe=False)
         for node in nodes:
             if 'lb_member' in node.data:
                 node.data.pop('lb_member')
                 node.store(oslo_context.get_current())
 
+        lb_data = cluster.data.get('loadbalancers', {})
+        if lb_data and isinstance(lb_data, dict):
+            lb_data.pop(self.id, None)
+            if lb_data:
+                cluster.data['loadbalancers'] = lb_data
+            else:
+                cluster.data.pop('loadbalancers')
+
         return True, reason
 
-    def post_op(self, cluster_id, action):
-        """Routine to be called after an action has been executed.
+    def _get_delete_candidates(self, cluster_id, action):
+        deletion = action.data.get('deletion', None)
+        # No deletion field in action.data which means no scaling
+        # policy or deletion policy is attached.
+        candidates = None
+        if deletion is None:
+            if action.action == consts.NODE_DELETE:
+                candidates = [action.node.id]
+                count = 1
+            elif action.action == consts.CLUSTER_DEL_NODES:
+                # Get candidates from action.input
+                candidates = action.inputs.get('candidates', [])
+                count = len(candidates)
+            elif action.action == consts.CLUSTER_RESIZE:
+                # Calculate deletion count based on action input
+                db_cluster = co.Cluster.get(action.context, cluster_id)
+                current = no.Node.count_by_cluster(action.context, cluster_id)
+                scaleutils.parse_resize_params(action, db_cluster, current)
+                if 'deletion' not in action.data:
+                    return []
+                else:
+                    count = action.data['deletion']['count']
+            else:  # action.action == consts.CLUSTER_SCALE_IN
+                count = 1
+        else:
+            count = deletion.get('count', 0)
+            candidates = deletion.get('candidates', None)
+
+        # Still no candidates available, pick count of nodes randomly
+        if candidates is None:
+            if count == 0:
+                return []
+            nodes = no.Node.get_all_by_cluster(action.context, cluster_id)
+            if count > len(nodes):
+                count = len(nodes)
+            candidates = scaleutils.nodes_by_random(nodes, count)
+            deletion_data = action.data.get('deletion', {})
+            deletion_data.update({
+                'count': len(candidates),
+                'candidates': candidates
+            })
+            action.data.update({'deletion': deletion_data})
+
+        return candidates
+
+    def pre_op(self, cluster_id, action):
+        """Routine to be called before an action has been executed.
 
         For this particular policy, we take this chance to update the pool
         maintained by the load-balancer.
@@ -279,26 +453,23 @@ class LoadBalancingPolicy(base.Policy):
         :param action: The action object that triggered this operation.
         :returns: Nothing.
         """
-        nodes_added = action.outputs.get('nodes_added', [])
-        nodes_removed = action.outputs.get('nodes_removed', [])
-        if ((len(nodes_added) == 0) and (len(nodes_removed) == 0)):
+
+        candidates = self._get_delete_candidates(cluster_id, action)
+        if len(candidates) == 0:
             return
 
-        db_cluster = db_api.cluster_get(action.context, cluster_id)
-        params = self._build_conn_params(db_cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        db_cluster = co.Cluster.get(action.context, cluster_id)
+        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
         policy_data = self._extract_policy_data(cp.data)
         lb_id = policy_data['loadbalancer']
         pool_id = policy_data['pool']
-        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
-        subnet = self.pool_spec.get(self.POOL_SUBNET)
 
-        # Remove nodes that have been deleted from lb pool
-        for node_id in nodes_removed:
-            node = node_mod.Node.load(action.context, node_id=node_id,
-                                      show_deleted=True)
+        # Remove nodes that will be deleted from lb pool
+        for node_id in candidates:
+            node = nm.Node.load(action.context, node_id=node_id)
             member_id = node.data.get('lb_member', None)
             if member_id is None:
                 LOG.warning(_LW('Node %(n)s not found in lb pool %(p)s.'),
@@ -312,10 +483,43 @@ class LoadBalancingPolicy(base.Policy):
                                           'node(s) from lb pool.')
                 return
 
+        return
+
+    def post_op(self, cluster_id, action):
+        """Routine to be called after an action has been executed.
+
+        For this particular policy, we take this chance to update the pool
+        maintained by the load-balancer.
+
+        :param cluster_id: The ID of the cluster on which a relevant action
+            has been executed.
+        :param action: The action object that triggered this operation.
+        :returns: Nothing.
+        """
+        # TODO(Yanyanhu): Need special handling for cross-az scenario
+        # which is supported by Neutron lbaas.
+        if action.action == consts.NODE_CREATE:
+            nodes_added = [action.node.id]
+        else:
+            creation = action.data.get('creation', None)
+            nodes_added = creation.get('nodes', []) if creation else []
+            if len(nodes_added) == 0:
+                return
+
+        db_cluster = co.Cluster.get(action.context, cluster_id)
+        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
+        cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
+                                               self.id)
+        policy_data = self._extract_policy_data(cp.data)
+        lb_id = policy_data['loadbalancer']
+        pool_id = policy_data['pool']
+        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
+        subnet = self.pool_spec.get(self.POOL_SUBNET)
+
         # Add new nodes to lb pool
         for node_id in nodes_added:
-            node = node_mod.Node.load(action.context, node_id=node_id,
-                                      show_deleted=True)
+            node = nm.Node.load(action.context, node_id=node_id)
             member_id = node.data.get('lb_member', None)
             if member_id:
                 LOG.warning(_LW('Node %(n)s already in lb pool %(p)s.'),
