@@ -12,6 +12,7 @@
 
 import mock
 
+from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_utils import timeutils as tu
 
 from senlin.common import exception
@@ -45,63 +46,45 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertEqual(0, cluster.desired_capacity)
         self.assertEqual('INIT', cluster.status)
         self.assertEqual('Just Initialized', cluster.status_reason)
-        self.assertIsNone(cluster.created_time)
-        self.assertIsNone(cluster.updated_time)
-        self.assertIsNone(cluster.deleted_time)
-        self.assertIsNotNone(cluster.init_time)
-        self.assertEqual({}, cluster.metadata)
+        self.assertIsNone(cluster.created_at)
+        self.assertIsNone(cluster.updated_at)
+        self.assertIsNotNone(cluster.init_at)
+        self.assertEqual({}, cluster.meta_data)
         self.assertIsNone(cluster.data)
 
     def test_cluster_get_returns_a_cluster(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
-        ret_cluster = db_api.cluster_get(self.ctx, cluster.id,
-                                         show_deleted=False)
+        ret_cluster = db_api.cluster_get(self.ctx, cluster.id)
         self.assertIsNotNone(ret_cluster)
         self.assertEqual(cluster.id, ret_cluster.id)
         self.assertEqual('db_test_cluster_name', ret_cluster.name)
 
     def test_cluster_get_not_found(self):
-        cluster = db_api.cluster_get(self.ctx, UUID1, show_deleted=False)
+        cluster = db_api.cluster_get(self.ctx, UUID1)
         self.assertIsNone(cluster)
 
     def test_cluster_get_from_different_project(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
         self.ctx.project = 'abc'
         ret_cluster = db_api.cluster_get(self.ctx, cluster.id,
-                                         show_deleted=False,
                                          project_safe=False)
         self.assertEqual(cluster.id, ret_cluster.id)
         self.assertEqual('db_test_cluster_name', ret_cluster.name)
 
-        cluster = db_api.cluster_get(self.ctx, cluster.id, show_deleted=False)
+        cluster = db_api.cluster_get(self.ctx, cluster.id)
         self.assertIsNone(cluster)
 
-    def test_cluster_get_show_deleted(self):
+    def test_cluster_get_with_admin_context(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
-        result = db_api.cluster_get(self.ctx, cluster.id)
-        self.assertEqual(cluster.id, result.id)
-
-        db_api.cluster_delete(self.ctx, cluster.id)
-        result = db_api.cluster_get(self.ctx, cluster.id)
-        self.assertIsNone(result)
-
-        result = db_api.cluster_get(self.ctx, cluster.id, show_deleted=True)
-        self.assertEqual(cluster.id, result.id)
-
-    def test_cluster_get_show_deleted_context(self):
-        cluster = shared.create_cluster(self.ctx, self.profile)
-
-        self.assertFalse(self.ctx.show_deleted)
-        result = db_api.cluster_get(self.ctx, cluster.id)
-        self.assertEqual(cluster.id, result.id)
-
-        db_api.cluster_delete(self.ctx, cluster.id)
-        result = db_api.cluster_get(self.ctx, cluster.id)
-        self.assertIsNone(result)
-
-        self.ctx.show_deleted = True
-        result = db_api.cluster_get(self.ctx, cluster.id)
-        self.assertEqual(cluster.id, result.id)
+        admin_ctx = utils.dummy_context(project='another-project',
+                                        is_admin=True)
+        ret_cluster = db_api.cluster_get(admin_ctx, cluster.id,
+                                         project_safe=True)
+        self.assertIsNone(ret_cluster)
+        ret_cluster = db_api.cluster_get(admin_ctx, cluster.id,
+                                         project_safe=False)
+        self.assertEqual(cluster.id, ret_cluster.id)
+        self.assertEqual('db_test_cluster_name', ret_cluster.name)
 
     def test_cluster_get_by_name(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
@@ -229,36 +212,22 @@ class DBAPIClusterTest(base.SenlinTestCase):
         clusters = db_api.cluster_get_all(self.ctx, project_safe=False)
         self.assertEqual(5, len(clusters))
 
-    def test_cluster_get_all_show_deleted(self):
-        clusters = [shared.create_cluster(self.ctx, self.profile)
-                    for x in range(3)]
+    def test_cluster_get_all_with_admin_context(self):
+        values = [
+            {'project': UUID1},
+            {'project': UUID1},
+            {'project': UUID2},
+            {'project': UUID2},
+            {'project': UUID2},
+        ]
+        [shared.create_cluster(self.ctx, self.profile, **v) for v in values]
 
-        results = db_api.cluster_get_all(self.ctx)
-        self.assertEqual(3, len(results))
-
-        db_api.cluster_delete(self.ctx, clusters[0].id)
-        results = db_api.cluster_get_all(self.ctx)
-        self.assertEqual(2, len(results))
-
-        results = db_api.cluster_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(3, len(results))
-
-    def test_cluster_get_all_show_nested(self):
-        cluster1 = shared.create_cluster(self.ctx, self.profile,
-                                         name='cluster1')
-        cluster2 = shared.create_cluster(self.ctx, self.profile,
-                                         name='cluster2',
-                                         parent=cluster1.id)
-
-        cl_db = db_api.cluster_get_all(self.ctx)
-        self.assertEqual(1, len(cl_db))
-        self.assertEqual(cluster1.id, cl_db[0].id)
-
-        cl_db = db_api.cluster_get_all(self.ctx, show_nested=True)
-        self.assertEqual(2, len(cl_db))
-        cl_ids = [s.id for s in cl_db]
-        self.assertIn(cluster1.id, cl_ids)
-        self.assertIn(cluster2.id, cl_ids)
+        admin_ctx = utils.dummy_context(project='another-project',
+                                        is_admin=True)
+        clusters = db_api.cluster_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(clusters))
+        clusters = db_api.cluster_get_all(admin_ctx, project_safe=False)
+        self.assertEqual(5, len(clusters))
 
     def test_cluster_get_all_with_filters(self):
         shared.create_cluster(self.ctx, self.profile, name='foo')
@@ -285,10 +254,10 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
     def test_cluster_get_all_default_sort_dir(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          init_time=tu.utcnow())
+                                          init_at=tu.utcnow(True))
                     for x in range(3)]
 
-        st_db = db_api.cluster_get_all(self.ctx, sort_dir='asc')
+        st_db = db_api.cluster_get_all(self.ctx)
         self.assertEqual(3, len(st_db))
         self.assertEqual(clusters[0].id, st_db[0].id)
         self.assertEqual(clusters[1].id, st_db[1].id)
@@ -296,30 +265,29 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
     def test_cluster_get_all_str_sort_keys(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          created_time=tu.utcnow())
+                                          created_at=tu.utcnow(True))
                     for x in range(3)]
 
-        st_db = db_api.cluster_get_all(self.ctx, sort_keys='created_time')
+        st_db = db_api.cluster_get_all(self.ctx, sort='created_at')
         self.assertEqual(3, len(st_db))
         self.assertEqual(clusters[0].id, st_db[0].id)
         self.assertEqual(clusters[1].id, st_db[1].id)
         self.assertEqual(clusters[2].id, st_db[2].id)
 
-    @mock.patch.object(db_api.utils, 'paginate_query')
+    @mock.patch.object(sa_utils, 'paginate_query')
     def test_cluster_get_all_filters_sort_keys(self, mock_paginate):
-        sort_keys = ['name', 'status', 'created_time',
-                     'updated_time', 'parent']
-        db_api.cluster_get_all(self.ctx, sort_keys=sort_keys)
+        sort = 'name,status,created_at,updated_at'
+        db_api.cluster_get_all(self.ctx, sort=sort)
 
         args = mock_paginate.call_args[0]
         used_sort_keys = set(args[3])
-        expected_keys = set(['name', 'status', 'created_time',
-                             'updated_time', 'id'])
+        expected_keys = set(['name', 'status', 'created_at',
+                             'updated_at', 'id'])
         self.assertEqual(expected_keys, used_sort_keys)
 
     def test_cluster_get_all_marker(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          created_time=tu.utcnow())
+                                          created_at=tu.utcnow(True))
                     for x in range(3)]
         cl_db = db_api.cluster_get_all(self.ctx, marker=clusters[1].id)
         self.assertEqual(1, len(cl_db))
@@ -331,12 +299,6 @@ class DBAPIClusterTest(base.SenlinTestCase):
         st_db = db_api.cluster_get_all(self.ctx, marker=uuid)
         self.assertEqual(3, len(st_db))
 
-    def test_cluster_get_all_doesnt_mutate_sort_keys(self):
-        [shared.create_cluster(self.ctx, self.profile) for x in range(3)]
-        sort_keys = ['id']
-        db_api.cluster_get_all(self.ctx, sort_keys=sort_keys)
-        self.assertEqual(['id'], sort_keys)
-
     def test_cluster_next_index(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
         cluster_id = cluster.id
@@ -361,16 +323,10 @@ class DBAPIClusterTest(base.SenlinTestCase):
         db_api.cluster_delete(self.ctx, clusters[0].id)
         cl_db = db_api.cluster_count_all(self.ctx)
         self.assertEqual(2, cl_db)
-        # show deleted
-        cl_db = db_api.cluster_count_all(self.ctx, show_deleted=True)
-        self.assertEqual(3, cl_db)
 
         db_api.cluster_delete(self.ctx, clusters[1].id)
         cl_db = db_api.cluster_count_all(self.ctx)
         self.assertEqual(1, cl_db)
-        # show deleted
-        cl_db = db_api.cluster_count_all(self.ctx, show_deleted=True)
-        self.assertEqual(3, cl_db)
 
     def test_cluster_count_all_with_regular_project(self):
         values = [
@@ -401,15 +357,22 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertEqual(5, db_api.cluster_count_all(self.ctx,
                                                      project_safe=False))
 
-    def test_cluster_count_all_show_nested(self):
-        cluster1 = shared.create_cluster(self.ctx, self.profile, name='c1')
-        shared.create_cluster(self.ctx, self.profile, name='c2',
-                              parent=cluster1.id)
+    def test_cluster_count_all_with_admin_context(self):
+        values = [
+            {'project': UUID1},
+            {'project': UUID1},
+            {'project': UUID2},
+            {'project': UUID2},
+            {'project': UUID2},
+        ]
+        [shared.create_cluster(self.ctx, self.profile, **v) for v in values]
 
-        results = db_api.cluster_count_all(self.ctx)
-        self.assertEqual(1, results)
-        results = db_api.cluster_count_all(self.ctx, show_nested=True)
-        self.assertEqual(2, results)
+        admin_ctx = utils.dummy_context(project='another-project',
+                                        is_admin=True)
+        self.assertEqual(0, db_api.cluster_count_all(admin_ctx,
+                                                     project_safe=True))
+        self.assertEqual(5, db_api.cluster_count_all(admin_ctx,
+                                                     project_safe=False))
 
     def test_cluster_count_all_with_filters(self):
         shared.create_cluster(self.ctx, self.profile, name='foo')
@@ -435,76 +398,9 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertEqual('update failed', cluster.status_reason)
         self.assertEqual(90, cluster.timeout)
 
-        self.assertRaises(exception.ClusterNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           db_api.cluster_update, self.ctx, UUID2, values)
 
-    def test_get_sort_keys_returns_empty_list_if_no_keys(self):
-        sort_keys = None
-        mapping = {}
-
-        filtered_keys = db_api._get_sort_keys(sort_keys, mapping)
-        self.assertEqual([], filtered_keys)
-
-    def test_get_sort_keys_whitelists_single_key(self):
-        sort_key = 'foo'
-        mapping = {'foo': 'Foo'}
-
-        filtered_keys = db_api._get_sort_keys(sort_key, mapping)
-        self.assertEqual(['Foo'], filtered_keys)
-
-    def test_get_sort_keys_whitelists_multiple_keys(self):
-        sort_keys = ['foo', 'bar', 'nope']
-        mapping = {'foo': 'Foo', 'bar': 'Bar'}
-
-        filtered_keys = db_api._get_sort_keys(sort_keys, mapping)
-        self.assertIn('Foo', filtered_keys)
-        self.assertIn('Bar', filtered_keys)
-        self.assertEqual(2, len(filtered_keys))
-
-    @mock.patch.object(db_api.utils, 'paginate_query')
-    def test_paginate_query_raises_invalid_sort_key(self, mock_paginate_query):
-        query = mock.Mock()
-        model = mock.Mock()
-
-        mock_paginate_query.side_effect = db_api.utils.InvalidSortKey()
-        self.assertRaises(exception.InvalidParameter, db_api._paginate_query,
-                          self.ctx, query, model, sort_keys=['foo'])
-
-    @mock.patch.object(db_api.utils, 'paginate_query')
-    @mock.patch.object(db_api, 'model_query')
-    def test_paginate_query_gets_model_marker(self, mock_query,
-                                              mock_paginate_query):
-        query = mock.Mock()
-        model = mock.Mock()
-        marker = mock.Mock()
-
-        mock_query_object = mock.Mock()
-        mock_query_object.get.return_value = 'real_marker'
-        mock_query.return_value = mock_query_object
-
-        db_api._paginate_query(self.ctx, query, model, marker=marker)
-        mock_query_object.get.assert_called_once_with(marker)
-        args, _ = mock_paginate_query.call_args
-        self.assertIn('real_marker', args)
-
-    @mock.patch.object(db_api.utils, 'paginate_query')
-    def test_paginate_query_default_sorts_dir_by_desc(self,
-                                                      mock_paginate_query):
-        query = mock.Mock()
-        model = mock.Mock()
-        db_api._paginate_query(self.ctx, query, model, sort_dir=None)
-        args, _ = mock_paginate_query.call_args
-        self.assertIn('asc', args)
-
-    @mock.patch.object(db_api.utils, 'paginate_query')
-    def test_paginate_query_uses_given_sort_plus_id(self,
-                                                    mock_paginate_query):
-        query = mock.Mock()
-        model = mock.Mock()
-        db_api._paginate_query(self.ctx, query, model, sort_keys=['name'])
-        args, _ = mock_paginate_query.call_args
-        self.assertIn(['name', 'id'], args)
-
     def test_nested_cluster_get_by_name(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster1')
@@ -519,34 +415,18 @@ class DBAPIClusterTest(base.SenlinTestCase):
         result = db_api.cluster_get_by_name(self.ctx, 'cluster2')
         self.assertIsNone(result)
 
-    def _deleted_cluster_existance(self, ctx, clusters, existing, deleted):
-        for s in existing:
-            self.assertIsNotNone(db_api.cluster_get(ctx, clusters[s].id,
-                                                    show_deleted=True))
-        for s in deleted:
-            self.assertIsNone(db_api.cluster_get(ctx, clusters[s].id,
-                                                 show_deleted=True))
-
     def test_cluster_delete(self):
         cluster = shared.create_cluster(self.ctx, self.profile)
         cluster_id = cluster.id
         node = shared.create_node(self.ctx, cluster, self.profile)
         db_api.cluster_delete(self.ctx, cluster_id)
 
-        self.assertIsNone(db_api.cluster_get(self.ctx, cluster_id,
-                                             show_deleted=False))
+        self.assertIsNone(db_api.cluster_get(self.ctx, cluster_id))
         res = db_api.node_get(self.ctx, node.id)
         self.assertIsNone(res)
-        self.assertRaises(exception.ClusterNotFound, db_api.cluster_delete,
+        self.assertRaises(exception.ResourceNotFound, db_api.cluster_delete,
                           self.ctx, cluster_id)
 
-        # Testing soft delete
-        ret_cluster = db_api.cluster_get(self.ctx, cluster_id,
-                                         show_deleted=True)
-        self.assertIsNotNone(ret_cluster)
-        self.assertEqual(cluster_id, ret_cluster.id)
-        self.assertEqual('db_test_cluster_name', ret_cluster.name)
-
         # Testing child nodes deletion
         res = db_api.node_get(self.ctx, node.id)
         self.assertIsNone(res)
@@ -563,8 +443,6 @@ class DBAPIClusterTest(base.SenlinTestCase):
             'user': self.ctx.user,
             'project': self.ctx.project,
             'spec': {'foo': 'bar'},
-            'level': 50,
-            'cooldown': 60,
             'data': None,
         }
         policy = db_api.policy_create(self.ctx, policy_data)
@@ -573,7 +451,6 @@ class DBAPIClusterTest(base.SenlinTestCase):
         # attach policy
         fields = {
             'enabled': True,
-            'priority': 77,
         }
         db_api.cluster_policy_attach(self.ctx, cluster_id, policy.id, fields)
         binding = db_api.cluster_policy_get(self.ctx, cluster_id, policy.id)
@@ -592,3 +469,34 @@ class DBAPIClusterTest(base.SenlinTestCase):
         # but the policy is not deleted
         result = db_api.policy_get(self.ctx, policy.id)
         self.assertIsNotNone(result)
+
+    def test_cluster_add_dependents(self):
+        cluster = shared.create_cluster(self.ctx, self.profile)
+        profile_id = 'profile1'
+        db_api.cluster_add_dependents(self.ctx, cluster.id, profile_id)
+        res = db_api.cluster_get(self.ctx, cluster.id)
+        self.assertEqual(['profile1'], res.dependents['profiles'])
+        deps = {}
+        cluster = shared.create_cluster(self.ctx, self.profile,
+                                        dependents=deps)
+        db_api.cluster_add_dependents(self.ctx, cluster.id, profile_id)
+        res = db_api.cluster_get(self.ctx, cluster.id)
+        deps = {'profiles': ['profile1']}
+        self.assertEqual(deps, res.dependents)
+        db_api.cluster_add_dependents(self.ctx, cluster.id, 'profile2')
+        res = db_api.cluster_get(self.ctx, cluster.id)
+        deps = {'profiles': ['profile1', 'profile2']}
+        self.assertEqual(deps, res.dependents)
+
+    def test_cluster_remove_dependents(self):
+        deps = {'profiles': ['profile1', 'profile2']}
+        cluster = shared.create_cluster(self.ctx, self.profile,
+                                        dependents=deps)
+        db_api.cluster_remove_dependents(self.ctx, cluster.id, 'profile1')
+        res = db_api.cluster_get(self.ctx, cluster.id)
+        deps = {'profiles': ['profile2']}
+        self.assertEqual(deps, res.dependents)
+        db_api.cluster_remove_dependents(self.ctx, cluster.id, 'profile2')
+        res = db_api.cluster_get(self.ctx, cluster.id)
+        deps = {}
+        self.assertEqual(deps, res.dependents)
