@@ -10,11 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
+import mock
+from oslo_db.sqlalchemy import utils as sa_utils
+from oslo_serialization import jsonutils
+from oslo_utils import timeutils as tu
 import six
 
-from oslo_utils import timeutils as tu
-
+from senlin.common import consts
 from senlin.common import exception
 from senlin.db.sqlalchemy import api as db_api
 from senlin.tests.unit.common import base
@@ -43,13 +45,12 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(UUID1, node.physical_id)
         self.assertEqual(1, node.index)
         self.assertIsNone(node.role)
-        self.assertIsNone(node.created_time)
-        self.assertIsNone(node.updated_time)
-        self.assertIsNone(node.deleted_time)
+        self.assertIsNone(node.created_at)
+        self.assertIsNone(node.updated_at)
         self.assertEqual('ACTIVE', node.status)
         self.assertEqual('create complete', node.status_reason)
-        self.assertEqual('{"foo": "123"}', json.dumps(node.metadata))
-        self.assertEqual('{"key1": "value1"}', json.dumps(node.data))
+        self.assertEqual('{"foo": "123"}', jsonutils.dumps(node.meta_data))
+        self.assertEqual('{"key1": "value1"}', jsonutils.dumps(node.data))
         self.assertEqual(self.cluster.id, node.cluster_id)
         self.assertEqual(self.profile.id, node.profile_id)
 
@@ -77,23 +78,15 @@ class DBAPINodeTest(base.SenlinTestCase):
         node = db_api.node_get(ctx_new, res.id, project_safe=False)
         self.assertIsNotNone(node)
 
-    def test_node_get_show_deleted(self):
+    def test_node_get_with_admin_context(self):
         res = shared.create_node(self.ctx, self.cluster, self.profile)
-        node_id = res.id
-        node = db_api.node_get(self.ctx, node_id)
+        admin_ctx = utils.dummy_context(project='a_different_project',
+                                        is_admin=True)
+        node = db_api.node_get(admin_ctx, res.id, project_safe=True)
+        self.assertIsNone(node)
+        node = db_api.node_get(admin_ctx, res.id, project_safe=False)
         self.assertIsNotNone(node)
 
-        db_api.node_delete(self.ctx, node_id)
-
-        node = db_api.node_get(self.ctx, node_id)
-        self.assertIsNone(node)
-
-        node = db_api.node_get(self.ctx, node_id, show_deleted=False)
-        self.assertIsNone(node)
-
-        node = db_api.node_get(self.ctx, node_id, show_deleted=True)
-        self.assertEqual(node_id, node.id)
-
     def test_node_get_by_name(self):
         shared.create_node(self.ctx, self.cluster, self.profile)
         node = db_api.node_get_by_name(self.ctx, 'test_node_name')
@@ -116,25 +109,6 @@ class DBAPINodeTest(base.SenlinTestCase):
                                       project_safe=False)
         self.assertIsNotNone(res)
 
-    def test_node_get_by_name_show_deleted(self):
-        node_name = 'test_node_name'
-        shared.create_node(self.ctx, self.cluster, self.profile,
-                           name=node_name)
-        node = db_api.node_get_by_name(self.ctx, node_name)
-        self.assertIsNotNone(node)
-
-        node_id = node.id
-        db_api.node_delete(self.ctx, node_id)
-
-        res = db_api.node_get_by_name(self.ctx, node_name)
-        self.assertIsNone(res)
-
-        res = db_api.node_get_by_name(self.ctx, node_name, show_deleted=False)
-        self.assertIsNone(res)
-
-        res = db_api.node_get_by_name(self.ctx, node_name, show_deleted=True)
-        self.assertEqual(node_id, res.id)
-
     def test_node_get_by_short_id(self):
         node_id1 = 'same-part-unique-part'
         node_id2 = 'same-part-part-unique'
@@ -169,25 +143,19 @@ class DBAPINodeTest(base.SenlinTestCase):
                                           project_safe=False)
         self.assertIsNotNone(res)
 
-    def test_node_get_by_short_id_show_deleted(self):
-        node_id = 'this-is-a-unique-id'
-        shared.create_node(self.ctx, None, self.profile, id=node_id)
+    def test_node_get_by_short_id_admin_context(self):
+        node_id = 'same-part-unique-part'
+        shared.create_node(self.ctx, None, self.profile,
+                           id=node_id, name='node-1')
 
-        res = db_api.node_get_by_short_id(self.ctx, node_id[:5])
-        self.assertEqual(node_id, res.id)
-        res = db_api.node_get_by_short_id(self.ctx, node_id[:7])
-        self.assertEqual(node_id, res.id)
-
-        db_api.node_delete(self.ctx, node_id)
-
-        res = db_api.node_get_by_short_id(self.ctx, node_id[:5])
+        admin_ctx = utils.dummy_context(project='a_different_project',
+                                        is_admin=True)
+        res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
+                                          project_safe=True)
         self.assertIsNone(res)
-        res = db_api.node_get_by_short_id(self.ctx, node_id[:5],
-                                          show_deleted=False)
-        self.assertIsNone(res)
-        res = db_api.node_get_by_short_id(self.ctx, node_id[:5],
-                                          show_deleted=True)
-        self.assertEqual(node_id, res.id)
+        res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
+                                          project_safe=False)
+        self.assertIsNotNone(res)
 
     def test_node_get_all(self):
         values = [{'name': 'node1'}, {'name': 'node2'}, {'name': 'node3'}]
@@ -199,6 +167,68 @@ class DBAPINodeTest(base.SenlinTestCase):
         names = [node.name for node in nodes]
         [self.assertIn(val['name'], names) for val in values]
 
+    def test_node_add_node_dependents(self):
+        node_id = 'host_node'
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, name='node-1')
+        db_api.node_add_dependents(self.ctx, node_id, 'NODE1')
+        node = db_api.node_get(self.ctx, node_id)
+        nodes = node.dependents['nodes']
+        self.assertEqual(['NODE1'], nodes)
+
+        db_api.node_add_dependents(self.ctx, node_id, 'NODE2')
+        new_node = db_api.node_get(self.ctx, node_id)
+        nodes = new_node.dependents['nodes']
+        self.assertEqual(['NODE1', 'NODE2'], nodes)
+
+    def test_node_add_profile_dependents(self):
+        node_id = 'host_node'
+        new_profile = shared.create_profile(self.ctx)
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, name='node-1')
+        db_api.node_add_dependents(self.ctx, node_id, new_profile.id,
+                                   'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        nodes = node.dependents['profiles']
+        self.assertEqual([new_profile.id], nodes)
+
+        new_profile_1 = shared.create_profile(self.ctx)
+        db_api.node_add_dependents(self.ctx, node_id, new_profile_1.id,
+                                   'profile')
+        new_node = db_api.node_get(self.ctx, node_id)
+        nodes = new_node.dependents['profiles']
+        self.assertEqual([new_profile.id, new_profile_1.id], nodes)
+
+    def test_node_remove_node_dependents(self):
+        node_id = 'host_node'
+        dependents = {'nodes': ['NODE1', 'NODE2']}
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, dependents=dependents)
+        db_api.node_remove_dependents(self.ctx, node_id, 'NODE1')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({'nodes': ['NODE2']}, dependents)
+
+        db_api.node_remove_dependents(self.ctx, node_id, 'NODE2', 'node')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({}, dependents)
+
+    def test_node_remove_profile_dependents(self):
+        node_id = 'host_node'
+        dependents = {'profiles': ['P1', 'P2']}
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, dependents=dependents)
+        db_api.node_remove_dependents(self.ctx, node_id, 'P1', 'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({'profiles': ['P2']}, dependents)
+
+        db_api.node_remove_dependents(self.ctx, node_id, 'P2', 'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({}, dependents)
+
     def test_node_get_all_with_cluster_id(self):
         values = [{'name': 'node1'}, {'name': 'node2'}, {'name': 'node3'}]
         for v in values:
@@ -211,27 +241,11 @@ class DBAPINodeTest(base.SenlinTestCase):
         names = [node.name for node in nodes]
         [self.assertIn(val['name'], names) for val in values]
 
-    def test_node_get_all_show_deleted(self):
-        values = [{'id': 'node1'}, {'id': 'node2'}, {'id': 'node3'}]
-        for v in values:
-            shared.create_node(self.ctx, self.cluster, self.profile, **v)
-
-        db_api.node_delete(self.ctx, 'node2')
-
-        nodes = db_api.node_get_all(self.ctx)
-        self.assertEqual(2, len(nodes))
-
-        nodes = db_api.node_get_all(self.ctx, show_deleted=False)
-        self.assertEqual(2, len(nodes))
-
-        nodes = db_api.node_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(3, len(nodes))
-
     def test_node_get_all_with_limit_marker(self):
         node_ids = ['node1', 'node2', 'node3']
         for v in node_ids:
             shared.create_node(self.ctx, self.cluster, self.profile,
-                               id=v, init_time=tu.utcnow())
+                               id=v, init_at=tu.utcnow(True))
 
         nodes = db_api.node_get_all(self.ctx, limit=1)
         self.assertEqual(1, len(nodes))
@@ -254,64 +268,56 @@ class DBAPINodeTest(base.SenlinTestCase):
         nodes = db_api.node_get_all(self.ctx, limit=1, marker='node1')
         self.assertEqual(1, len(nodes))
 
-    def test_node_get_all_used_sort_keys(self):
+    @mock.patch.object(sa_utils, 'paginate_query')
+    def test_node_get_all_used_sort_keys(self, mock_paginate):
         node_ids = ['node1', 'node2', 'node3']
         for v in node_ids:
             shared.create_node(self.ctx, self.cluster, self.profile, id=v)
 
-        mock_paginate = self.patchobject(db_api.utils, 'paginate_query')
-        sort_keys = ['index', 'name', 'created_time', 'updated_time',
-                     'deleted_time', 'status']
+        sort = ','.join(consts.NODE_SORT_KEYS)
 
-        db_api.node_get_all(self.ctx, sort_keys=sort_keys)
+        db_api.node_get_all(self.ctx, sort=sort)
         args = mock_paginate.call_args[0]
         used_sort_keys = set(args[3])
-        expected_keys = set(['index', 'name', 'created_time', 'updated_time',
-                             'deleted_time', 'status', 'id'])
+        sort_keys = consts.NODE_SORT_KEYS
+        sort_keys.append('id')
+        expected_keys = set(sort_keys)
         self.assertEqual(expected_keys, used_sort_keys)
 
-    def test_node_get_all_sort_keys_wont_change(self):
-        sort_keys = ['id']
-        db_api.node_get_all(self.ctx, sort_keys=sort_keys)
-        self.assertEqual(['id'], sort_keys)
-
-    def test_node_get_all_sort_keys_and_dir(self):
+    def test_node_get_all_sorting(self):
         values = [{'id': '001', 'name': 'node1', 'status': 'ACTIVE'},
                   {'id': '002', 'name': 'node3', 'status': 'ERROR'},
                   {'id': '003', 'name': 'node2', 'status': 'UPDATING'}]
         for v in values:
             shared.create_node(self.ctx, self.cluster, self.profile, **v)
 
-        nodes = db_api.node_get_all(self.ctx, sort_keys=['name', 'status'],
-                                    sort_dir='asc')
+        nodes = db_api.node_get_all(self.ctx, sort='name,status')
         self.assertEqual(3, len(nodes))
         # Sorted by name
         self.assertEqual('001', nodes[0].id)
         self.assertEqual('003', nodes[1].id)
         self.assertEqual('002', nodes[2].id)
 
-        nodes = db_api.node_get_all(self.ctx, sort_keys=['status', 'name'],
-                                    sort_dir='asc')
+        nodes = db_api.node_get_all(self.ctx, sort='status,name')
         self.assertEqual(3, len(nodes))
         # Sorted by statuses (ascending)
         self.assertEqual('001', nodes[0].id)
         self.assertEqual('002', nodes[1].id)
         self.assertEqual('003', nodes[2].id)
 
-        nodes = db_api.node_get_all(self.ctx, sort_keys=['status', 'name'],
-                                    sort_dir='desc')
+        nodes = db_api.node_get_all(self.ctx, sort='status:desc,name:desc')
         self.assertEqual(3, len(nodes))
         # Sorted by statuses (descending)
         self.assertEqual('003', nodes[0].id)
         self.assertEqual('002', nodes[1].id)
         self.assertEqual('001', nodes[2].id)
 
-    def test_node_get_all_default_sort_dir(self):
+    def test_node_get_all_default_sorting(self):
         nodes = [shared.create_node(self.ctx, None, self.profile,
-                                    init_time=tu.utcnow())
+                                    init_at=tu.utcnow(True))
                  for x in range(3)]
 
-        results = db_api.node_get_all(self.ctx, sort_dir='asc')
+        results = db_api.node_get_all(self.ctx)
         self.assertEqual(3, len(results))
         self.assertEqual(nodes[0].id, results[0].id)
         self.assertEqual(nodes[1].id, results[1].id)
@@ -354,6 +360,17 @@ class DBAPINodeTest(base.SenlinTestCase):
         results = db_api.node_get_all(self.ctx, project_safe=True)
         self.assertEqual(0, len(results))
 
+    def test_node_get_all_with_admin_context(self):
+        shared.create_node(self.ctx, None, self.profile, name='node1')
+        shared.create_node(self.ctx, None, self.profile, name='node2')
+
+        admin_ctx = utils.dummy_context(project='a_different_project',
+                                        is_admin=True)
+        results = db_api.node_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(results))
+        results = db_api.node_get_all(admin_ctx, project_safe=False)
+        self.assertEqual(2, len(results))
+
     def test_node_get_by_cluster(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
 
@@ -367,10 +384,16 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(set([node1.id, node2.id]),
                          set([nodes[0].id, nodes[1].id]))
 
-        nodes = db_api.node_get_all_by_cluster(self.ctx, None)
+        # retrieve orphan nodes
+        nodes = db_api.node_get_all_by_cluster(self.ctx, '')
         self.assertEqual(1, len(nodes))
         self.assertEqual(node0.id, nodes[0].id)
 
+        # retrieve all nodes
+        nodes = db_api.node_get_all_by_cluster(self.ctx, None)
+        self.assertEqual(4, len(nodes))
+        self.assertEqual(node0.id, nodes[0].id)
+
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster1.id)
         self.assertEqual(1, len(nodes))
         self.assertEqual(node3.id, nodes[0].id)
@@ -393,93 +416,87 @@ class DBAPINodeTest(base.SenlinTestCase):
                                                project_safe=False)
         self.assertEqual(2, len(nodes))
 
-    def test_node_get_by_cluster_show_deleted(self):
-        node0 = shared.create_node(self.ctx, self.cluster, self.profile)
+    def test_node_get_by_cluster_admin_context(self):
+        shared.create_cluster(self.ctx, self.profile)
+
         node1 = shared.create_node(self.ctx, self.cluster, self.profile)
+        node2 = shared.create_node(self.ctx, self.cluster, self.profile)
 
-        nodes = db_api.node_get_all_by_cluster(self.ctx,
-                                               self.cluster.id)
+        admin_ctx = utils.dummy_context(project='a_different_project',
+                                        is_admin=True)
+        nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id)
+        self.assertEqual(0, len(nodes))
+        nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id,
+                                               project_safe=False)
         self.assertEqual(2, len(nodes))
-        self.assertEqual(set([node0.id, node1.id]),
+        self.assertEqual(set([node1.id, node2.id]),
                          set([nodes[0].id, nodes[1].id]))
 
-        db_api.node_delete(self.ctx, node1.id)
-        nodes = db_api.node_get_all_by_cluster(self.ctx,
-                                               self.cluster.id)
-        self.assertEqual(1, len(nodes))
-        self.assertEqual(node0.id, nodes[0].id)
+    def test_node_count_by_cluster(self):
+        shared.create_cluster(self.ctx, self.profile)
 
-        nodes = db_api.node_get_all_by_cluster(self.ctx,
-                                               self.cluster.id,
-                                               show_deleted=True)
-        self.assertEqual(2, len(nodes))
-        self.assertEqual(set([node0.id, node1.id]),
-                         set([nodes[0].id, nodes[1].id]))
+        shared.create_node(self.ctx, self.cluster, self.profile)
+        shared.create_node(self.ctx, self.cluster, self.profile)
+
+        res = db_api.node_count_by_cluster(self.ctx, self.cluster.id)
+        self.assertEqual(2, res)
+
+    def test_node_count_by_cluster_with_filters(self):
+        shared.create_cluster(self.ctx, self.profile)
 
-    def test_node_get_by_name_and_cluster(self):
-        node_name = 'test_node_007'
         shared.create_node(self.ctx, self.cluster, self.profile,
-                           name=node_name)
-        node = db_api.node_get_by_name_and_cluster(self.ctx,
-                                                   node_name,
-                                                   self.cluster.id)
-        self.assertIsNotNone(node)
-        self.assertEqual(node_name, node.name)
-        self.assertEqual(self.cluster.id, node.cluster_id)
-
-        node = db_api.node_get_by_name_and_cluster(self.ctx, 'not-exist',
-                                                   self.cluster.id)
-        self.assertIsNone(node)
-
-        node = db_api.node_get_by_name_and_cluster(self.ctx, node_name,
-                                                   'BogusClusterID')
-        self.assertIsNone(node)
-
-    def test_node_get_by_name_and_cluster_diff_project(self):
-        node_name = 'test_node_007'
+                           status='ACTIVE')
         shared.create_node(self.ctx, self.cluster, self.profile,
-                           name=node_name)
-        node = db_api.node_get_by_name_and_cluster(self.ctx,
-                                                   node_name,
-                                                   self.cluster.id)
-        self.assertIsNotNone(node)
+                           status='ERROR')
 
+        res = db_api.node_count_by_cluster(self.ctx, self.cluster.id,
+                                           status='ACTIVE')
+        self.assertEqual(1, res)
+        res = db_api.node_count_by_cluster(self.ctx, self.cluster.id,
+                                           status='ERROR')
+        self.assertEqual(1, res)
+
+    def test_node_count_by_cluster_diff_project(self):
         ctx_new = utils.dummy_context(project='a_different_project')
-        node = db_api.node_get_by_name_and_cluster(ctx_new,
-                                                   node_name,
-                                                   self.cluster.id)
-        self.assertIsNone(node)
-        node = db_api.node_get_by_name_and_cluster(ctx_new,
-                                                   node_name,
-                                                   self.cluster.id,
-                                                   project_safe=False)
-        self.assertIsNotNone(node)
+        shared.create_cluster(self.ctx, self.profile)
 
-    def test_node_get_by_physical_id(self):
-        shared.create_node(self.ctx, self.cluster, self.profile,
-                           physical_id=UUID1)
+        shared.create_node(self.ctx, self.cluster, self.profile)
+        shared.create_node(self.ctx, self.cluster, self.profile)
 
-        node = db_api.node_get_by_physical_id(self.ctx, UUID1)
-        self.assertIsNotNone(node)
-        self.assertEqual(UUID1, node.physical_id)
+        res = db_api.node_count_by_cluster(ctx_new, self.cluster.id)
+        self.assertEqual(0, res)
 
-        node = db_api.node_get_by_physical_id(self.ctx, UUID2)
-        self.assertIsNone(node)
+        res = db_api.node_count_by_cluster(ctx_new, self.cluster.id,
+                                           project_safe=False)
+        self.assertEqual(2, res)
 
-    def test_node_get_by_physical_id_diff_project(self):
-        shared.create_node(self.ctx, self.cluster, self.profile,
-                           physical_id=UUID1)
+    def test_node_count_by_cluster_admin_context(self):
+        shared.create_cluster(self.ctx, self.profile)
+        shared.create_node(self.ctx, self.cluster, self.profile)
+        shared.create_node(self.ctx, self.cluster, self.profile)
 
-        node = db_api.node_get_by_physical_id(self.ctx, UUID1)
-        self.assertIsNotNone(node)
-        self.assertEqual(UUID1, node.physical_id)
+        admin_ctx = utils.dummy_context(project='a_different_project',
+                                        is_admin=True)
+        res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
+                                           project_safe=True)
+        self.assertEqual(0, res)
+        res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
+                                           project_safe=False)
+        self.assertEqual(2, res)
 
-        ctx_new = utils.dummy_context(project='a_different_project')
-        node = db_api.node_get_by_physical_id(ctx_new, UUID1)
-        self.assertIsNone(node)
-        node = db_api.node_get_by_physical_id(ctx_new, UUID1,
-                                              project_safe=False)
-        self.assertIsNotNone(node)
+    def test_nodeids_by_cluster(self):
+        node0 = shared.create_node(self.ctx, None, self.profile)
+        node1 = shared.create_node(self.ctx, self.cluster, self.profile)
+        node2 = shared.create_node(self.ctx, self.cluster, self.profile)
+
+        results = db_api.node_ids_by_cluster(self.ctx, self.cluster.id)
+        self.assertEqual(2, len(results))
+        self.assertEqual(set([node1.id, node2.id]), set(results))
+
+        # retrieve orphan nodes
+        results = db_api.node_ids_by_cluster(self.ctx, '')
+        self.assertEqual(1, len(results))
+        self.assertEqual(node0.id, results[0])
 
     def test_node_update(self):
         node = shared.create_node(self.ctx, self.cluster, self.profile)
@@ -497,10 +514,10 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_update_not_found(self):
         new_attributes = {'name': 'new_name'}
-        ex = self.assertRaises(exception.NodeNotFound,
+        ex = self.assertRaises(exception.ResourceNotFound,
                                db_api.node_update,
                                self.ctx, 'BogusId', new_attributes)
-        self.assertEqual('The node (BogusId) could not be found.',
+        self.assertEqual("The node 'BogusId' could not be found.",
                          six.text_type(ex))
 
     def test_node_update_cluster_status_updated(self):
@@ -529,24 +546,25 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_migrate_from_none(self):
         node_orphan = shared.create_node(self.ctx, None, self.profile)
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node = db_api.node_migrate(self.ctx, node_orphan.id, self.cluster.id,
-                                   timestamp)
+                                   timestamp, 'NEW-ROLE')
         cluster = db_api.cluster_get(self.ctx, self.cluster.id)
-        self.assertEqual(timestamp, node.updated_time)
+        self.assertEqual(timestamp, node.updated_at)
         self.assertEqual(self.cluster.id, node.cluster_id)
         self.assertEqual(2, cluster.next_index)
         nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id)
         self.assertEqual(1, len(nodes))
+        self.assertEqual('NEW-ROLE', nodes[0].role)
 
     def test_node_migrate_to_none(self):
         node = shared.create_node(self.ctx, self.cluster, self.profile)
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, None, timestamp)
-        self.assertEqual(timestamp, node_new.updated_time)
-        self.assertIsNone(node_new.cluster_id)
+        self.assertEqual(timestamp, node_new.updated_at)
+        self.assertEqual('', node_new.cluster_id)
         nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id)
         self.assertEqual(0, len(nodes))
 
@@ -559,17 +577,21 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(1, len(nodes))
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster2.id)
         self.assertEqual(0, len(nodes))
+        # Refresh cluster1 and cluster2
+        cluster1 = db_api.cluster_get(self.ctx, cluster1.id)
+        cluster2 = db_api.cluster_get(self.ctx, cluster2.id)
         self.assertEqual(2, cluster1.next_index)
         self.assertEqual(1, cluster2.next_index)
 
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, cluster2.id,
                                        timestamp)
         cluster1 = db_api.cluster_get(self.ctx, cluster1.id)
         cluster2 = db_api.cluster_get(self.ctx, cluster2.id)
-        self.assertEqual(timestamp, node_new.updated_time)
+        self.assertEqual(timestamp, node_new.updated_at)
         self.assertEqual(cluster2.id, node_new.cluster_id)
+        self.assertIsNone(node_new.role)
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster1.id)
         self.assertEqual(0, len(nodes))
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster2.id)
@@ -578,14 +600,15 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(2, cluster2.next_index)
 
         # Migrate it back!
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, cluster1.id,
-                                       timestamp)
+                                       timestamp, 'FAKE-ROLE')
         cluster1 = db_api.cluster_get(self.ctx, cluster1.id)
         cluster2 = db_api.cluster_get(self.ctx, cluster2.id)
-        self.assertEqual(timestamp, node_new.updated_time)
+        self.assertEqual(timestamp, node_new.updated_at)
         self.assertEqual(cluster1.id, node_new.cluster_id)
+        self.assertEqual('FAKE-ROLE', node_new.role)
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster1.id)
         self.assertEqual(1, len(nodes))
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster2.id)
