@@ -10,8 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import mock
+from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_utils import timeutils as tu
 
+from senlin.common import consts
 from senlin.common import exception
 from senlin.db.sqlalchemy import api as db_api
 from senlin.tests.unit.common import base
@@ -40,8 +43,6 @@ class DBAPIPolicyTest(base.SenlinTestCase):
             'project': self.ctx.project,
             'domain': self.ctx.domain,
             'spec': sample_spec,
-            'level': 50,
-            'cooldown': 60,
             'data': None,
         }
 
@@ -55,8 +56,6 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertIsNotNone(policy)
         self.assertEqual(data['name'], policy.name)
         self.assertEqual(data['type'], policy.type)
-        self.assertEqual(data['cooldown'], policy.cooldown)
-        self.assertEqual(data['level'], policy.level)
         self.assertEqual(data['spec'], policy.spec)
         self.assertEqual(10, policy.spec['max_size'])
         self.assertIsNone(policy.data)
@@ -69,8 +68,6 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertIsNotNone(retobj)
         self.assertEqual(data['name'], retobj.name)
         self.assertEqual(data['type'], retobj.type)
-        self.assertEqual(data['cooldown'], retobj.cooldown)
-        self.assertEqual(data['level'], retobj.level)
         self.assertEqual(data['spec'], retobj.spec)
         self.assertEqual(10, retobj.spec['max_size'])
         self.assertIsNone(retobj.data)
@@ -86,33 +83,21 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertIsNotNone(res)
         self.assertEqual(policy.id, res.id)
 
+    def test_policy_get_admin_context(self):
+        data = self.new_policy_data()
+        policy = db_api.policy_create(self.ctx, data)
+
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        res = db_api.policy_get(admin_ctx, policy.id, project_safe=True)
+        self.assertIsNone(res)
+        res = db_api.policy_get(admin_ctx, policy.id, project_safe=False)
+        self.assertIsNotNone(res)
+
     def test_policy_get_not_found(self):
         retobj = db_api.policy_get(self.ctx, 'BogusID')
         self.assertIsNone(retobj)
 
-    def test_policy_get_show_deleted(self):
-        data = self.new_policy_data()
-        policy_id = db_api.policy_create(self.ctx, data).id
-
-        # check created
-        policy = db_api.policy_get(self.ctx, policy_id)
-        self.assertIsNotNone(policy)
-
-        db_api.policy_delete(self.ctx, policy_id)
-
-        # default equivalent to false
-        policy = db_api.policy_get(self.ctx, policy_id)
-        self.assertIsNone(policy)
-
-        # explicit false
-        policy = db_api.policy_get(self.ctx, policy_id, show_deleted=False)
-        self.assertIsNone(policy)
-
-        # explicit true
-        policy = db_api.policy_get(self.ctx, policy_id, show_deleted=True)
-        self.assertIsNotNone(policy)
-        self.assertEqual(policy_id, policy.id)
-
     def test_policy_get_by_name(self):
         policy_name = 'my_best_policy'
         data = self.new_policy_data(name=policy_name)
@@ -145,28 +130,6 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertIsNotNone(res)
         self.assertEqual(policy.id, res.id)
 
-    def test_policy_get_by_name_show_deleted(self):
-        policy_name = 'my_best_policy'
-        data = self.new_policy_data(name=policy_name)
-        policy_id = db_api.policy_create(self.ctx, data).id
-
-        db_api.policy_delete(self.ctx, policy_id)
-
-        # default case
-        policy = db_api.policy_get_by_name(self.ctx, policy_name)
-        self.assertIsNone(policy)
-
-        # explicit false
-        policy = db_api.policy_get_by_name(self.ctx, policy_name,
-                                           show_deleted=False)
-        self.assertIsNone(policy)
-
-        # explicit true
-        policy = db_api.policy_get_by_name(self.ctx, policy_name,
-                                           show_deleted=True)
-        self.assertIsNotNone(policy)
-        self.assertEqual(policy_id, policy.id)
-
     def test_policy_get_by_short_id(self):
         policy_ids = ['same-part-unique-part',
                       'same-part-part-unique']
@@ -225,26 +188,17 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         for spec in specs:
             self.assertIn(spec['name'], names)
 
-        # test show_deleted here
         db_api.policy_delete(self.ctx, policies[1].id)
 
         # after delete one of them
         policies = db_api.policy_get_all(self.ctx)
         self.assertEqual(1, len(policies))
 
-        policies = db_api.policy_get_all(self.ctx, show_deleted=False)
-        self.assertEqual(1, len(policies))
-
-        policies = db_api.policy_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(2, len(policies))
-
         # after delete both policies
         db_api.policy_delete(self.ctx, policies[0].id)
 
         policies = db_api.policy_get_all(self.ctx)
         self.assertEqual(0, len(policies))
-        policies = db_api.policy_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(2, len(policies))
 
     def test_policy_get_all_diff_project(self):
         specs = [
@@ -262,11 +216,28 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         policies = db_api.policy_get_all(new_ctx, project_safe=False)
         self.assertEqual(2, len(policies))
 
+    def test_policy_get_all_admin_context(self):
+        specs = [
+            {'name': 'policy_short', 'cooldown': '10'},
+            {'name': 'policy_long', 'cooldown': '100'},
+        ]
+
+        for spec in specs:
+            data = self.new_policy_data(**spec)
+            db_api.policy_create(self.ctx, data)
+
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        policies = db_api.policy_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(policies))
+        policies = db_api.policy_get_all(admin_ctx, project_safe=False)
+        self.assertEqual(2, len(policies))
+
     def test_policy_get_all_with_limit_marker(self):
         ids = ['policy1', 'policy2', 'policy3']
         for pid in ids:
-            timestamp = tu.utcnow()
-            data = self.new_policy_data(id=pid, created_time=timestamp)
+            timestamp = tu.utcnow(True)
+            data = self.new_policy_data(id=pid, created_at=timestamp)
             db_api.policy_create(self.ctx, data)
 
         # different limit settings
@@ -293,69 +264,60 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         policies = db_api.policy_get_all(self.ctx, limit=1, marker='policy1')
         self.assertEqual(1, len(policies))
 
-    def test_policy_get_all_used_sort_keys(self):
+    @mock.patch.object(sa_utils, 'paginate_query')
+    def test_policy_get_all_used_sort_keys(self, mock_paginate):
         ids = ['policy1', 'policy2', 'policy3']
         for pid in ids:
             data = self.new_policy_data(id=pid)
             db_api.policy_create(self.ctx, data)
 
-        mock_paginate = self.patchobject(db_api.utils, 'paginate_query')
-        sort_keys = ['type', 'name', 'level', 'cooldown', 'created_time',
-                     'updated_time', 'deleted_time']
-
-        db_api.policy_get_all(self.ctx, sort_keys=sort_keys)
+        sort_keys = consts.POLICY_SORT_KEYS
+        db_api.policy_get_all(self.ctx, sort=','.join(sort_keys))
 
         args = mock_paginate.call_args[0]
         used_sort_keys = set(args[3])
-        expected_keys = set(['id', 'type', 'name', 'level', 'cooldown',
-                             'created_time', 'updated_time', 'deleted_time'])
+        sort_keys.append('id')
+        expected_keys = set(sort_keys)
         self.assertEqual(expected_keys, used_sort_keys)
 
-    def test_policy_get_all_sort_keys_wont_change(self):
-        sort_keys = ['id']
-        db_api.policy_get_all(self.ctx, sort_keys=sort_keys)
-        self.assertEqual(['id'], sort_keys)
-
-    def test_policy_get_all_sort_keys_and_dir(self):
-        values = [{'id': '001', 'name': 'policy1', 'level': 50},
-                  {'id': '002', 'name': 'policy3', 'level': 20},
-                  {'id': '003', 'name': 'policy2', 'level': 40}]
+    def test_policy_get_all_sorting(self):
+        values = [{'id': '001', 'name': 'policy1'},
+                  {'id': '002', 'name': 'policy3'},
+                  {'id': '003', 'name': 'policy2'}]
 
         for v in values:
+            v['created_at'] = tu.utcnow(True)
             data = self.new_policy_data(**v)
             db_api.policy_create(self.ctx, data)
 
-        policies = db_api.policy_get_all(self.ctx, sort_keys=['name', 'level'],
-                                         sort_dir='asc')
-        self.assertEqual(3, len(policies))
         # Sorted by name
+        policies = db_api.policy_get_all(self.ctx, sort='name')
+        self.assertEqual(3, len(policies))
         self.assertEqual('001', policies[0].id)
         self.assertEqual('003', policies[1].id)
         self.assertEqual('002', policies[2].id)
 
-        policies = db_api.policy_get_all(self.ctx, sort_keys=['level', 'name'],
-                                         sort_dir='asc')
+        # Sorted by created_at and name (ascending)
+        policies = db_api.policy_get_all(self.ctx, sort='created_at,name')
+        self.assertEqual(3, len(policies))
+        self.assertEqual('001', policies[0].id)
+        self.assertEqual('002', policies[1].id)
+        self.assertEqual('003', policies[2].id)
+
+        # Sorted by name (descending)
+        policies = db_api.policy_get_all(self.ctx, sort='name:desc')
         self.assertEqual(3, len(policies))
-        # Sorted by levels (ascending)
         self.assertEqual('002', policies[0].id)
         self.assertEqual('003', policies[1].id)
         self.assertEqual('001', policies[2].id)
 
-        policies = db_api.policy_get_all(self.ctx, sort_keys=['level', 'name'],
-                                         sort_dir='desc')
-        self.assertEqual(3, len(policies))
-        # Sorted by statuses (descending)
-        self.assertEqual('001', policies[0].id)
-        self.assertEqual('003', policies[1].id)
-        self.assertEqual('002', policies[2].id)
-
-    def test_policy_get_all_default_sort_dir(self):
+    def test_policy_get_all_default_sorting(self):
         policies = []
         for x in range(3):
-            data = self.new_policy_data(created_time=tu.utcnow())
+            data = self.new_policy_data(created_at=tu.utcnow(True))
             policies.append(db_api.policy_create(self.ctx, data))
 
-        results = db_api.policy_get_all(self.ctx, sort_dir='asc')
+        results = db_api.policy_get_all(self.ctx)
         self.assertEqual(3, len(results))
         self.assertEqual(policies[0].id, results[0].id)
         self.assertEqual(policies[1].id, results[1].id)
@@ -389,12 +351,9 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         another_policy = {
             'name': 'new_scaling_policy',
             'type': 'ScalingPolicy',
-            'cooldown': 11,
-            'level': 60,
             'spec': {
                 'min_size': 5,
                 'max_size': 15,
-                'pause_time': 'PT10M',
             }
         }
         old_data = self.new_policy_data()
@@ -406,10 +365,9 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertEqual(old_policy.id, new_policy.id)
         self.assertEqual(new_data['name'], new_policy.name)
         self.assertEqual('new_scaling_policy', new_policy.name)
-        self.assertEqual(11, new_policy.cooldown)
 
     def test_policy_update_not_found(self):
-        self.assertRaises(exception.PolicyNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           db_api.policy_update,
                           self.ctx, 'BogusID', {})
 
@@ -433,11 +391,10 @@ class DBAPIPolicyTest(base.SenlinTestCase):
 
         fields = {
             'enabled': True,
-            'level': 50
         }
         db_api.cluster_policy_attach(self.ctx, self.cluster.id, policy.id,
                                      fields)
-        self.assertRaises(exception.ResourceBusyError,
+        self.assertRaises(exception.EResourceBusy,
                           db_api.policy_delete,
                           self.ctx, policy.id)
 
