@@ -10,10 +10,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import mock
+from oslo_db.sqlalchemy import utils as sa_utils
+from oslo_utils import timeutils as tu
 import six
 
-from oslo_utils import timeutils as tu
-
+from senlin.common import consts
 from senlin.common import exception
 from senlin.db.sqlalchemy import api as db_api
 from senlin.engine import parser
@@ -34,7 +36,6 @@ class DBAPIProfileTest(base.SenlinTestCase):
         self.assertEqual(data['name'], profile.name)
         self.assertEqual(data['type'], profile.type)
         self.assertEqual(data['spec'], profile.spec)
-        self.assertEqual(data['permission'], profile.permission)
 
     def test_profile_get(self):
         profile = shared.create_profile(self.ctx)
@@ -52,33 +53,19 @@ class DBAPIProfileTest(base.SenlinTestCase):
         self.assertIsNotNone(res)
         self.assertEqual(profile.id, res.id)
 
+    def test_profile_get_admin_context(self):
+        profile = shared.create_profile(self.ctx)
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        res = db_api.profile_get(admin_ctx, profile.id, project_safe=True)
+        self.assertIsNone(res)
+        res = db_api.profile_get(admin_ctx, profile.id, project_safe=False)
+        self.assertIsNotNone(res)
+
     def test_profile_get_not_found(self):
         profile = db_api.profile_get(self.ctx, 'BogusProfileID')
         self.assertIsNone(profile)
 
-    def test_profile_get_show_deleted(self):
-        profile_id = shared.create_profile(self.ctx).id
-
-        # check created
-        profile = db_api.profile_get(self.ctx, profile_id)
-        self.assertIsNotNone(profile)
-
-        # Now, delete it
-        db_api.profile_delete(self.ctx, profile_id)
-
-        # default equivalent to false
-        profile = db_api.profile_get(self.ctx, profile_id)
-        self.assertIsNone(profile)
-
-        # explicit false
-        profile = db_api.profile_get(self.ctx, profile_id, show_deleted=False)
-        self.assertIsNone(profile)
-
-        # explicit true
-        profile = db_api.profile_get(self.ctx, profile_id, show_deleted=True)
-        self.assertIsNotNone(profile)
-        self.assertEqual(profile_id, profile.id)
-
     def test_profile_get_by_name(self):
         profile_name = 'my_best_profile'
 
@@ -110,28 +97,6 @@ class DBAPIProfileTest(base.SenlinTestCase):
         self.assertIsNotNone(res)
         self.assertEqual(profile_name, res.name)
 
-    def test_profile_get_by_name_show_deleted(self):
-        profile_name = 'my_best_profile'
-
-        profile_id = shared.create_profile(self.ctx, name=profile_name).id
-
-        db_api.profile_delete(self.ctx, profile_id)
-
-        # default case
-        profile = db_api.profile_get_by_name(self.ctx, profile_name)
-        self.assertIsNone(profile)
-
-        # explicit false
-        profile = db_api.profile_get_by_name(self.ctx, profile_name,
-                                             show_deleted=False)
-        self.assertIsNone(profile)
-
-        # explicit true
-        profile = db_api.profile_get_by_name(self.ctx, profile_name,
-                                             show_deleted=True)
-        self.assertIsNotNone(profile)
-        self.assertEqual(profile_id, profile.id)
-
     def test_profile_get_by_short_id(self):
         profile_ids = ['same-part-unique-part',
                        'same-part-part-unique']
@@ -185,26 +150,17 @@ class DBAPIProfileTest(base.SenlinTestCase):
         for pid in ids:
             self.assertIn(pid, profile_ids)
 
-        # test show_deleted here
         db_api.profile_delete(self.ctx, profiles[1].id)
 
         # after delete one of them
         profiles = db_api.profile_get_all(self.ctx)
         self.assertEqual(1, len(profiles))
 
-        profiles = db_api.profile_get_all(self.ctx, show_deleted=False)
-        self.assertEqual(1, len(profiles))
-
-        profiles = db_api.profile_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(2, len(profiles))
-
         # after delete both profiles
         db_api.profile_delete(self.ctx, profiles[0].id)
 
         profiles = db_api.profile_get_all(self.ctx)
         self.assertEqual(0, len(profiles))
-        profiles = db_api.profile_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(2, len(profiles))
 
     def test_profile_get_all_diff_project(self):
         ids = ['profile1', 'profile2']
@@ -217,11 +173,23 @@ class DBAPIProfileTest(base.SenlinTestCase):
         profiles = db_api.profile_get_all(new_ctx, project_safe=False)
         self.assertEqual(2, len(profiles))
 
+    def test_profile_get_all_admin_context(self):
+        ids = ['profile1', 'profile2']
+        for pid in ids:
+            shared.create_profile(self.ctx, id=pid)
+
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        profiles = db_api.profile_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(profiles))
+        profiles = db_api.profile_get_all(admin_ctx, project_safe=False)
+        self.assertEqual(2, len(profiles))
+
     def test_profile_get_all_with_limit_marker(self):
         ids = ['profile1', 'profile2', 'profile3']
         for pid in ids:
-            timestamp = tu.utcnow()
-            shared.create_profile(self.ctx, id=pid, created_time=timestamp)
+            timestamp = tu.utcnow(True)
+            shared.create_profile(self.ctx, id=pid, created_at=timestamp)
 
         # different limit settings
         profiles = db_api.profile_get_all(self.ctx, limit=1)
@@ -247,29 +215,20 @@ class DBAPIProfileTest(base.SenlinTestCase):
         profiles = db_api.profile_get_all(self.ctx, limit=1, marker='profile1')
         self.assertEqual(1, len(profiles))
 
-    def test_profile_get_all_used_sort_keys(self):
+    @mock.patch.object(sa_utils, 'paginate_query')
+    def test_profile_get_all_used_sort_keys(self, mock_paginate):
         ids = ['profile1', 'profile2', 'profile3']
         for pid in ids:
             shared.create_profile(self.ctx, id=pid)
 
-        mock_paginate = self.patchobject(db_api.utils, 'paginate_query')
-        sort_keys = ['type', 'name', 'permission', 'metadata',
-                     'created_time', 'updated_time', 'deleted_time']
-
-        db_api.profile_get_all(self.ctx, sort_keys=sort_keys)
+        sort_keys = consts.PROFILE_SORT_KEYS
+        db_api.profile_get_all(self.ctx, sort=','.join(sort_keys))
 
         args = mock_paginate.call_args[0]
-        used_sort_keys = set(args[3])
-        expected_keys = set(['id', 'type', 'name', 'permission',
-                             'created_time', 'updated_time', 'deleted_time'])
-        self.assertEqual(expected_keys, used_sort_keys)
+        sort_keys.append('id')
+        self.assertEqual(set(sort_keys), set(args[3]))
 
-    def test_profile_get_all_sort_keys_wont_change(self):
-        sort_keys = ['id']
-        db_api.profile_get_all(self.ctx, sort_keys=sort_keys)
-        self.assertEqual(['id'], sort_keys)
-
-    def test_profile_get_all_sort_keys_and_dir(self):
+    def test_profile_get_all_sorting(self):
         values = [{'id': '001', 'name': 'profile1', 'type': 'C'},
                   {'id': '002', 'name': 'profile3', 'type': 'B'},
                   {'id': '003', 'name': 'profile2', 'type': 'A'}]
@@ -277,37 +236,35 @@ class DBAPIProfileTest(base.SenlinTestCase):
         for v in values:
             shared.create_profile(self.ctx, **v)
 
-        profiles = db_api.profile_get_all(self.ctx, sort_keys=['name', 'type'],
-                                          sort_dir='asc')
+        # Sorted by name,type
+        profiles = db_api.profile_get_all(self.ctx, sort='name,type')
         self.assertEqual(3, len(profiles))
-        # Sorted by name
         self.assertEqual('001', profiles[0].id)
         self.assertEqual('003', profiles[1].id)
         self.assertEqual('002', profiles[2].id)
 
-        profiles = db_api.profile_get_all(self.ctx, sort_keys=['type', 'name'],
-                                          sort_dir='asc')
+        # Sorted by type,name (ascending)
+        profiles = db_api.profile_get_all(self.ctx, sort='type,name')
         self.assertEqual(3, len(profiles))
-        # Sorted by levels (ascending)
         self.assertEqual('003', profiles[0].id)
         self.assertEqual('002', profiles[1].id)
         self.assertEqual('001', profiles[2].id)
 
-        profiles = db_api.profile_get_all(self.ctx, sort_keys=['type', 'name'],
-                                          sort_dir='desc')
+        # Sorted by type,name (descending)
+        profiles = db_api.profile_get_all(self.ctx, sort='type:desc,name:desc')
         self.assertEqual(3, len(profiles))
-        # Sorted by statuses (descending)
         self.assertEqual('001', profiles[0].id)
         self.assertEqual('002', profiles[1].id)
         self.assertEqual('003', profiles[2].id)
 
-    def test_profile_get_all_default_sort_dir(self):
+    def test_profile_get_all_default_sorting(self):
         profiles = []
         for x in range(3):
-            profile = shared.create_profile(self.ctx, created_time=tu.utcnow())
+            profile = shared.create_profile(self.ctx,
+                                            created_at=tu.utcnow(True))
             profiles.append(profile)
 
-        results = db_api.profile_get_all(self.ctx, sort_dir='asc')
+        results = db_api.profile_get_all(self.ctx)
         self.assertEqual(3, len(results))
         self.assertEqual(profiles[0].id, results[0].id)
         self.assertEqual(profiles[1].id, results[1].id)
@@ -350,7 +307,6 @@ class DBAPIProfileTest(base.SenlinTestCase):
                     'myfile': 'new contents',
                 },
             },
-            'permission': 'yyyyxxxx',
         }
 
         old_profile = shared.create_profile(self.ctx)
@@ -362,7 +318,7 @@ class DBAPIProfileTest(base.SenlinTestCase):
         self.assertEqual('test_profile_name_2', new_profile.name)
 
     def test_profile_update_not_found(self):
-        self.assertRaises(exception.ProfileNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           db_api.profile_update,
                           self.ctx, 'BogusID', {})
 
@@ -384,9 +340,9 @@ class DBAPIProfileTest(base.SenlinTestCase):
         cluster = shared.create_cluster(self.ctx, profile)
 
         profile_id = profile.id
-        ex = self.assertRaises(exception.ResourceBusyError,
+        ex = self.assertRaises(exception.EResourceBusy,
                                db_api.profile_delete, self.ctx, profile_id)
-        self.assertEqual('The profile (%s) is busy now.' % profile_id,
+        self.assertEqual("The profile '%s' is busy now." % profile_id,
                          six.text_type(ex))
 
         db_api.cluster_delete(self.ctx, cluster.id)
@@ -397,9 +353,9 @@ class DBAPIProfileTest(base.SenlinTestCase):
         node = shared.create_node(self.ctx, None, profile)
 
         profile_id = profile.id
-        ex = self.assertRaises(exception.ResourceBusyError,
+        ex = self.assertRaises(exception.EResourceBusy,
                                db_api.profile_delete, self.ctx, profile_id)
-        self.assertEqual('The profile (%s) is busy now.' % profile_id,
+        self.assertEqual("The profile '%s' is busy now." % profile_id,
                          six.text_type(ex))
 
         db_api.node_delete(self.ctx, node.id)
