@@ -10,19 +10,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import uuid
-
+import datetime
 import mock
 
+from oslo_config import cfg
+from oslo_utils import timeutils
+from oslo_utils import uuidutils
+
 from senlin.common import consts
 from senlin.common import context
 from senlin.common import messaging as rpc_messaging
 from senlin.engine import service
+from senlin.objects import service as service_obj
 from senlin.tests.unit.common import base
 
 
 @mock.patch('senlin.engine.dispatcher.Dispatcher')
-@mock.patch('senlin.engine.health_manager.Health_Manager')
+@mock.patch('senlin.engine.health_manager.HealthManager')
 @mock.patch('oslo_messaging.Target')
 class EngineBasicTest(base.SenlinTestCase):
 
@@ -30,7 +34,9 @@ class EngineBasicTest(base.SenlinTestCase):
 
         super(EngineBasicTest, self).setUp()
         self.eng = service.EngineService('host-a', 'topic-a')
-        self.gen_id = self.patchobject(uuid, 'uuid4', return_value='1234')
+        self.fake_id = '4db0a14c-dc10-4131-8ed6-7573987ce9b0'
+        self.gen_id = self.patchobject(uuidutils, 'generate_uuid',
+                                       return_value=self.fake_id)
 
         self.fake_rpc_server = mock.Mock()
         self.get_rpc = self.patchobject(rpc_messaging, 'get_rpc_server',
@@ -49,7 +55,7 @@ class EngineBasicTest(base.SenlinTestCase):
         self.eng.start()
 
         self.gen_id.assert_called_once_with()
-        self.assertEqual('1234', self.eng.engine_id)
+        self.assertEqual(self.fake_id, self.eng.engine_id)
         self.assertIsNotNone(self.eng.TG)
 
         mock_disp_cls.assert_called_once_with(self.eng,
@@ -61,21 +67,23 @@ class EngineBasicTest(base.SenlinTestCase):
 
         mock_hm_cls.assert_called_once_with(self.eng,
                                             self.eng.health_mgr_topic,
-                                            consts.RPC_API_VERSION,
-                                            self.eng.TG)
+                                            consts.RPC_API_VERSION)
         self.assertEqual(mock_hm, self.eng.health_mgr)
         mock_hm.start.assert_called_once_with()
 
         mock_msg_cls.assert_called_once_with(version=consts.RPC_API_VERSION,
                                              server=self.eng.host,
                                              topic=self.eng.topic)
-        self.assertEqual(mock_target, self.eng.target)
 
-        self.get_rpc.assert_called_once_with(mock_target, self.eng)
+        self.get_rpc.assert_called_once_with(mock_target, self.eng,
+                                             serializer=mock.ANY)
+
         self.assertEqual(self.fake_rpc_server, self.eng._rpc_server)
         self.fake_rpc_server.start.assert_called_once_with()
 
-    def test_engine_stop(self, mock_msg_cls, mock_hm_cls, mock_disp_cls):
+    @mock.patch.object(service_obj.Service, 'delete')
+    def test_engine_stop(self, mock_delete, mock_msg_cls, mock_hm_cls,
+                         mock_disp_cls):
         mock_disp = mock_disp_cls.return_value
         mock_hm = mock_hm_cls.return_value
         self.eng.start()
@@ -88,6 +96,8 @@ class EngineBasicTest(base.SenlinTestCase):
         mock_disp.stop.assert_called_once_with()
         mock_hm.stop.assert_called_once_with()
 
+        mock_delete.assert_called_once_with(mock.ANY, self.fake_id)
+
     def test_engine_stop_with_exception(self, mock_msg_cls, mock_hm_cls,
                                         mock_disp_cls):
         mock_disp = mock_disp_cls.return_value
@@ -103,3 +113,53 @@ class EngineBasicTest(base.SenlinTestCase):
 
         mock_disp.stop.assert_called_once_with()
         mock_hm.stop.assert_called_once_with()
+
+
+class EngineStatusTest(base.SenlinTestCase):
+
+    def setUp(self):
+        super(EngineStatusTest, self).setUp()
+        self.eng = service.EngineService('host-a', 'topic-a')
+        fake_id = '4db0a14c-dc10-4131-8ed6-7573987ce9b0'
+        self.gen_id = self.patchobject(uuidutils, 'generate_uuid',
+                                       return_value=fake_id)
+
+        self.fake_rpc_server = mock.Mock()
+        self.get_rpc = self.patchobject(rpc_messaging, 'get_rpc_server',
+                                        return_value=self.fake_rpc_server)
+
+    @mock.patch.object(service_obj.Service, 'create')
+    @mock.patch.object(service_obj.Service, 'update')
+    def test_service_manage_report_create(self, mock_update, mock_create):
+        mock_update.return_value = None
+
+        self.eng.service_manage_report()
+
+        mock_create.assert_called_once_with(
+            mock.ANY, self.eng.engine_id, self.eng.host, 'senlin-engine',
+            self.eng.topic)
+
+    @mock.patch.object(service_obj.Service, 'update')
+    def test_service_manage_report_update(self, mock_update):
+        mock_update.return_value = mock.Mock()
+        self.eng.service_manage_report()
+        mock_update.assert_called_once_with(mock.ANY, self.eng.engine_id)
+
+    @mock.patch.object(service_obj.Service, 'update')
+    def test_service_manage_report_error(self, mock_update):
+        mock_update.side_effect = [Exception]
+        self.eng.service_manage_report()
+        mock_update.assert_called_once_with(mock.ANY, self.eng.engine_id)
+        expect_str = 'Service %s update failed' % self.eng.engine_id
+        self.assertIn(expect_str, self.LOG.output)
+
+    @mock.patch.object(service_obj.Service, 'gc_by_engine')
+    @mock.patch.object(service_obj.Service, 'get_all')
+    @mock.patch.object(service_obj.Service, 'delete')
+    def test__service_manage_cleanup(self, mock_delete, mock_get_all, mock_gc):
+        delta = datetime.timedelta(seconds=2 * cfg.CONF.periodic_interval)
+        ages_a_go = timeutils.utcnow(True) - delta
+        mock_get_all.return_value = [{'id': 'foo', 'updated_at': ages_a_go}]
+        self.eng._service_manage_cleanup()
+        mock_delete.assert_called_once_with(mock.ANY, 'foo')
+        mock_gc.assert_called_once_with(mock.ANY, 'foo')
