@@ -13,8 +13,10 @@
 import eventlet
 import mock
 from oslo_config import cfg
+from oslo_context import context as oslo_context
 from oslo_service import threadgroup
 
+from senlin.db import api as db_api
 from senlin.engine.actions import base as actionm
 from senlin.engine import scheduler
 from senlin.tests.unit.common import base
@@ -41,9 +43,9 @@ class DummyThreadGroup(object):
     def stop_timers(self):
         pass
 
-    def add_thread(self, callback, *args, **kwargs):
-        callback = args[1]
-        self.threads.append(callback)
+    def add_thread(self, callback, cnxt, trace, func, *args, **kwargs):
+        # callback here is _start_with_trace, func is the 'real' callback
+        self.threads.append(func)
         return DummyThread()
 
     def stop(self, graceful=False):
@@ -83,22 +85,102 @@ class SchedulerTest(base.SenlinTestCase):
         tgm = scheduler.ThreadGroupManager()
         tgm.start(f)
 
-        mock_group.add_thread.assert_called_once_with(f)
+        mock_group.add_thread.assert_called_once_with(
+            tgm._start_with_trace,
+            oslo_context.get_current(),
+            None, f)
 
-    def test_start_action(self):
+    @mock.patch.object(db_api, 'action_acquire_random_ready')
+    @mock.patch.object(db_api, 'action_acquire')
+    def test_start_action(self, mock_action_acquire,
+                          mock_action_acquire_1st):
+        mock_group = mock.Mock()
+        self.mock_tg.return_value = mock_group
+        action = mock.Mock()
+        action.id = '0123'
+        mock_action_acquire.return_value = action
+        mock_action_acquire_1st.return_value = None
+
+        tgm = scheduler.ThreadGroupManager()
+        tgm.start_action('4567', '0123')
+
+        mock_group.add_thread.assert_called_once_with(
+            tgm._start_with_trace,
+            oslo_context.get_current(),
+            None, actionm.ActionProc,
+            tgm.db_session, '0123')
+        mock_thread = mock_group.add_thread.return_value
+        self.assertEqual(mock_thread, tgm.workers['0123'])
+        mock_thread.link.assert_called_once_with(mock.ANY, '0123')
+
+    @mock.patch.object(db_api, 'action_acquire_random_ready')
+    def test_start_action_no_action_id(self, mock_acquire_action):
+        mock_action = mock.Mock()
+        mock_action.id = '0123'
+        mock_action.action = 'CLUSTER_CREATE'
+        mock_acquire_action.side_effect = [mock_action, None]
         mock_group = mock.Mock()
         self.mock_tg.return_value = mock_group
 
         tgm = scheduler.ThreadGroupManager()
-        tgm.start_action('0123', '4567')
+        tgm.start_action('4567')
 
-        mock_group.add_thread.assert_called_once_with(actionm.ActionProc,
-                                                      tgm.db_session,
-                                                      '0123', '4567')
+        mock_group.add_thread.assert_called_once_with(
+            tgm._start_with_trace,
+            oslo_context.get_current(),
+            None, actionm.ActionProc,
+            tgm.db_session, '0123')
         mock_thread = mock_group.add_thread.return_value
         self.assertEqual(mock_thread, tgm.workers['0123'])
         mock_thread.link.assert_called_once_with(mock.ANY, '0123')
 
+    @mock.patch.object(scheduler, 'sleep')
+    @mock.patch.object(db_api, 'action_acquire_random_ready')
+    def test_start_action_batch_control(self, mock_acquire_action, mock_sleep):
+        mock_action1 = mock.Mock()
+        mock_action1.id = 'ID1'
+        mock_action1.action = 'NODE_CREATE'
+        mock_action2 = mock.Mock()
+        mock_action2.id = 'ID2'
+        mock_action2.action = 'CLUSTER_CREATE'
+        mock_action3 = mock.Mock()
+        mock_action3.id = 'ID3'
+        mock_action3.action = 'NODE_DELETE'
+        mock_acquire_action.side_effect = [mock_action1, mock_action2,
+                                           mock_action3, None]
+        mock_group = mock.Mock()
+        self.mock_tg.return_value = mock_group
+        cfg.CONF.set_override('max_actions_per_batch', 1, enforce_type=True)
+        cfg.CONF.set_override('batch_interval', 3, enforce_type=True)
+
+        tgm = scheduler.ThreadGroupManager()
+        tgm.start_action('4567')
+
+        mock_sleep.assert_called_once_with(3)
+
+    @mock.patch.object(db_api, 'action_acquire_random_ready')
+    @mock.patch.object(db_api, 'action_acquire')
+    def test_start_action_failed_locking_action(self, mock_acquire_action,
+                                                mock_acquire_action_1st):
+        mock_acquire_action.return_value = None
+        mock_acquire_action_1st.return_value = None
+        mock_group = mock.Mock()
+        self.mock_tg.return_value = mock_group
+
+        tgm = scheduler.ThreadGroupManager()
+        res = tgm.start_action('4567', '0123')
+        self.assertIsNone(res)
+
+    @mock.patch.object(db_api, 'action_acquire_random_ready')
+    def test_start_action_no_action_ready(self, mock_acquire_action):
+        mock_acquire_action.return_value = None
+        mock_group = mock.Mock()
+        self.mock_tg.return_value = mock_group
+
+        tgm = scheduler.ThreadGroupManager()
+        res = tgm.start_action('4567')
+        self.assertIsNone(res)
+
     def test_cancel_action(self):
         mock_action = mock.Mock()
         mock_load = self.patchobject(actionm.Action, 'load',
@@ -106,7 +188,8 @@ class SchedulerTest(base.SenlinTestCase):
         tgm = scheduler.ThreadGroupManager()
         tgm.cancel_action('action0123')
 
-        mock_load.assert_called_once_with(tgm.db_session, 'action0123')
+        mock_load.assert_called_once_with(tgm.db_session, 'action0123',
+                                          project_safe=False)
         mock_action.signal.assert_called_once_with(mock_action.SIG_CANCEL)
 
     def test_suspend_action(self):
@@ -116,7 +199,8 @@ class SchedulerTest(base.SenlinTestCase):
         tgm = scheduler.ThreadGroupManager()
         tgm.suspend_action('action0123')
 
-        mock_load.assert_called_once_with(tgm.db_session, 'action0123')
+        mock_load.assert_called_once_with(tgm.db_session, 'action0123',
+                                          project_safe=False)
         mock_action.signal.assert_called_once_with(mock_action.SIG_SUSPEND)
 
     def test_resume_action(self):
@@ -126,7 +210,8 @@ class SchedulerTest(base.SenlinTestCase):
         tgm = scheduler.ThreadGroupManager()
         tgm.resume_action('action0123')
 
-        mock_load.assert_called_once_with(tgm.db_session, 'action0123')
+        mock_load.assert_called_once_with(tgm.db_session, 'action0123',
+                                          project_safe=False)
         mock_action.signal.assert_called_once_with(mock_action.SIG_RESUME)
 
     def test_add_timer(self):
