@@ -12,18 +12,24 @@
 
 import mock
 from oslo_context import context as oslo_ctx
+from oslo_utils import timeutils
 import six
 
+from senlin.common import consts
 from senlin.common import context as senlin_ctx
 from senlin.common import exception
 from senlin.common import schema
-from senlin.db import api as db_api
+from senlin.common import utils as common_utils
 from senlin.engine import environment
 from senlin.engine import parser
-from senlin.policies import base as policy_base
+from senlin.objects import credential as co
+from senlin.objects import policy as po
+from senlin.policies import base as pb
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
+UUID1 = 'aa5f86b8-e52b-4f2b-828a-4c14c770938d'
+UUID2 = '2c5139a6-24ba-4a6f-bd53-a268f61536de'
 
 sample_policy = """
   type: senlin.policy.dummy
@@ -34,7 +40,7 @@ sample_policy = """
 """
 
 
-class DummyPolicy(policy_base.Policy):
+class DummyPolicy(pb.Policy):
     VERSION = '1.0'
 
     properties_schema = {
@@ -57,15 +63,15 @@ class TestPolicyBase(base.SenlinTestCase):
     def setUp(self):
         super(TestPolicyBase, self).setUp()
         self.ctx = utils.dummy_context()
-        environment.global_env().register_policy('senlin.policy.dummy',
+        environment.global_env().register_policy('senlin.policy.dummy-1.0',
                                                  DummyPolicy)
         self.spec = parser.simple_parse(sample_policy)
 
     def _create_policy(self, policy_name, policy_id=None):
-        policy = policy_base.Policy(policy_name, self.spec,
-                                    user=self.ctx.user,
-                                    project=self.ctx.project,
-                                    domain=self.ctx.domain)
+        policy = pb.Policy(policy_name, self.spec,
+                           user=self.ctx.user,
+                           project=self.ctx.project,
+                           domain=self.ctx.domain)
         if policy_id:
             policy.id = policy_id
 
@@ -76,16 +82,14 @@ class TestPolicyBase(base.SenlinTestCase):
             'name': 'test-policy',
             'type': 'senlin.policy.dummy-1.0',
             'spec': self.spec,
+            'created_at': timeutils.utcnow(True),
             'user': self.ctx.user,
             'project': self.ctx.project,
             'domain': self.ctx.domain,
-            'level': policy_base.SHOULD,
-            'cooldown': 0,
-            'data': {}
         }
 
         values.update(kwargs)
-        return db_api.policy_create(self.ctx, values)
+        return po.Policy.create(self.ctx, values)
 
     def test_init(self):
         policy = self._create_policy('test-policy')
@@ -97,12 +101,9 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertEqual(self.ctx.user, policy.user)
         self.assertEqual(self.ctx.project, policy.project)
         self.assertEqual(self.ctx.domain, policy.domain)
-        self.assertEqual(policy_base.SHOULD, policy.level)
-        self.assertEqual(0, policy.cooldown)
         self.assertEqual({}, policy.data)
-        self.assertIsNone(policy.created_time)
-        self.assertIsNone(policy.updated_time)
-        self.assertIsNone(policy.deleted_time)
+        self.assertIsNone(policy.created_at)
+        self.assertIsNone(policy.updated_at)
         self.assertTrue(policy.singleton)
 
         spec_data = policy.spec_data
@@ -119,13 +120,13 @@ class TestPolicyBase(base.SenlinTestCase):
             'properties': '',
         }
 
-        self.assertRaises(exception.PolicyTypeNotFound,
-                          policy_base.Policy,
+        self.assertRaises(exception.ResourceNotFound,
+                          pb.Policy,
                           'test-policy', bad_spec)
 
     def test_load(self):
-        policy = self._create_db_policy()
-        result = policy_base.Policy.load(self.ctx, policy.id)
+        policy = utils.create_policy(self.ctx, UUID1)
+        result = pb.Policy.load(self.ctx, policy.id)
 
         self.assertEqual(policy.id, result.id)
         self.assertEqual(policy.name, result.name)
@@ -133,97 +134,59 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertEqual(policy.user, result.user)
         self.assertEqual(policy.project, result.project)
         self.assertEqual(policy.domain, result.domain)
-        self.assertEqual(policy.level, result.level)
-        self.assertEqual(policy.cooldown, result.cooldown)
         self.assertEqual(policy.spec, result.spec)
         self.assertEqual(policy.data, result.data)
         self.assertEqual({'key1': 'value1', 'key2': 2}, result.properties)
 
-        self.assertEqual(policy.created_time, result.created_time)
-        self.assertEqual(policy.updated_time, result.updated_time)
-        self.assertEqual(policy.deleted_time, result.deleted_time)
+        self.assertEqual(policy.created_at, result.created_at)
+        self.assertEqual(policy.updated_at, result.updated_at)
+
+    def test_load_with_policy(self):
+        policy = utils.create_policy(self.ctx, UUID1)
+        expected = pb.Policy.load(self.ctx, policy.id)
+
+        res = pb.Policy.load(self.ctx, db_policy=policy)
+
+        self.assertIsNotNone(res)
+        self.assertEqual(expected.id, res.id)
 
     def test_load_diff_project(self):
-        policy = self._create_db_policy()
+        policy = utils.create_policy(self.ctx, UUID1)
 
         new_ctx = utils.dummy_context(project='a-different-project')
-        self.assertRaises(exception.PolicyNotFound,
-                          policy_base.Policy.load,
+        self.assertRaises(exception.ResourceNotFound,
+                          pb.Policy.load,
                           new_ctx, policy.id, None)
 
-        res = policy_base.Policy.load(new_ctx, policy.id, project_safe=False)
+        res = pb.Policy.load(new_ctx, policy.id, project_safe=False)
         self.assertIsNotNone(res)
         self.assertEqual(policy.id, res.id)
 
     def test_load_not_found(self):
-        ex = self.assertRaises(exception.PolicyNotFound,
-                               policy_base.Policy.load,
+        ex = self.assertRaises(exception.ResourceNotFound,
+                               pb.Policy.load,
                                self.ctx, 'fake-policy', None)
-        self.assertEqual('The policy (fake-policy) could not be found.',
+        self.assertEqual("The policy 'fake-policy' could not be found.",
                          six.text_type(ex))
 
-        ex = self.assertRaises(exception.PolicyNotFound,
-                               policy_base.Policy.load,
+        ex = self.assertRaises(exception.ResourceNotFound,
+                               pb.Policy.load,
                                self.ctx, None, None)
-        self.assertEqual('The policy (None) could not be found.',
+        self.assertEqual("The policy 'None' could not be found.",
                          six.text_type(ex))
 
-    def test_load_all(self):
-        result = policy_base.Policy.load_all(self.ctx)
-        self.assertEqual([], list(result))
-
-        policy1 = self._create_db_policy(name='policy-1', id='ID1')
-        policy2 = self._create_db_policy(name='policy-2', id='ID2')
-
-        result = policy_base.Policy.load_all(self.ctx)
-        policies = list(result)
-        self.assertEqual(2, len(policies))
-        self.assertEqual(policy1.id, policies[0].id)
-        self.assertEqual(policy2.id, policies[1].id)
-
-    def test_load_all_diff_project(self):
-        self._create_db_policy(name='policy-1', id='ID1')
-        self._create_db_policy(name='policy-2', id='ID2')
-
-        new_ctx = utils.dummy_context(project='a-different-project')
-        res = policy_base.Policy.load_all(new_ctx)
-        self.assertEqual(0, len(list(res)))
-        res = policy_base.Policy.load_all(new_ctx, project_safe=False)
-        self.assertEqual(2, len(list(res)))
-
-    @mock.patch.object(db_api, 'policy_get_all')
-    def test_load_all_with_params(self, mock_get):
-        mock_get.return_value = []
-
-        res = list(policy_base.Policy.load_all(self.ctx))
-        self.assertEqual([], res)
-        mock_get.assert_called_once_with(self.ctx, limit=None, marker=None,
-                                         sort_keys=None, sort_dir=None,
-                                         filters=None, show_deleted=False,
-                                         project_safe=True)
-        mock_get.reset_mock()
-
-        res = list(policy_base.Policy.load_all(
-            self.ctx, limit=1, marker='MARKER', sort_keys=['K1'],
-            sort_dir='asc', filters={'level': 30}, show_deleted=False))
-        self.assertEqual([], res)
-        mock_get.assert_called_once_with(
-            self.ctx, limit=1, marker='MARKER', sort_keys=['K1'],
-            sort_dir='asc', filters={'level': 30}, show_deleted=False,
-            project_safe=True)
-
     def test_delete(self):
-        policy = self._create_db_policy()
+        policy = utils.create_policy(self.ctx, UUID1)
         policy_id = policy.id
 
-        res = policy_base.Policy.delete(self.ctx, policy_id)
+        res = pb.Policy.delete(self.ctx, policy_id)
         self.assertIsNone(res)
-        self.assertRaises(exception.PolicyNotFound,
-                          policy_base.Policy.load,
+        self.assertRaises(exception.ResourceNotFound,
+                          pb.Policy.load,
                           self.ctx, policy_id, None)
 
     def test_delete_not_found(self):
-        result = policy_base.Policy.delete(self.ctx, 'bogus')
+        result = pb.Policy.delete(self.ctx, 'bogus')
         self.assertIsNone(result)
 
     def test_store_for_create(self):
@@ -234,7 +197,7 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertIsNotNone(policy_id)
         self.assertEqual(policy_id, policy.id)
 
-        result = db_api.policy_get(self.ctx, policy_id)
+        result = po.Policy.get(self.ctx, policy_id)
 
         self.assertIsNotNone(result)
         self.assertEqual('test-policy', result.name)
@@ -243,14 +206,11 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertEqual(policy.user, result.user)
         self.assertEqual(policy.project, result.project)
         self.assertEqual(policy.domain, result.domain)
-        self.assertEqual(policy.level, result.level)
-        self.assertEqual(policy.cooldown, result.cooldown)
         self.assertEqual(policy.spec, result.spec)
         self.assertEqual(policy.data, result.data)
 
-        self.assertIsNotNone(result.created_time)
-        self.assertIsNone(result.updated_time)
-        self.assertIsNone(result.deleted_time)
+        self.assertIsNotNone(result.created_at)
+        self.assertIsNone(result.updated_at)
 
     def test_store_for_update(self):
         policy = self._create_policy('test-policy')
@@ -261,21 +221,17 @@ class TestPolicyBase(base.SenlinTestCase):
 
         # do an update
         policy.name = 'test-policy-1'
-        policy.level = 10
-        policy.cooldown = 60
         policy.data = {'kk': 'vv'}
 
         new_id = policy.store(self.ctx)
         self.assertEqual(policy_id, new_id)
 
-        result = db_api.policy_get(self.ctx, policy_id)
+        result = po.Policy.get(self.ctx, policy_id)
         self.assertIsNotNone(result)
         self.assertEqual('test-policy-1', result.name)
-        self.assertEqual(10, result.level)
-        self.assertEqual(60, result.cooldown)
         self.assertEqual({'kk': 'vv'}, policy.data)
-        self.assertIsNotNone(policy.created_time)
-        self.assertIsNotNone(policy.updated_time)
+        self.assertIsNotNone(policy.created_at)
+        self.assertIsNotNone(policy.updated_at)
 
     def test_to_dict(self):
         policy = self._create_policy('test-policy')
@@ -289,15 +245,12 @@ class TestPolicyBase(base.SenlinTestCase):
             'project': policy.project,
             'domain': policy.domain,
             'spec': policy.spec,
-            'level': policy.level,
-            'cooldown': policy.cooldown,
             'data': policy.data,
-            'created_time': policy.created_time.isoformat(),
-            'updated_time': None,
-            'deleted_time': None,
+            'created_at': common_utils.isotime(policy.created_at),
+            'updated_at': None,
         }
 
-        result = policy_base.Policy.load(self.ctx, policy_id=policy.id)
+        result = pb.Policy.load(self.ctx, policy_id=policy.id)
         self.assertEqual(expected, result.to_dict())
 
     def test_get_schema(self):
@@ -305,14 +258,14 @@ class TestPolicyBase(base.SenlinTestCase):
             'key1': {
                 'default': 'value1',
                 'description': 'first key',
-                'readonly': False,
                 'required': False,
+                'updatable': False,
                 'type': 'String'
             },
             'key2': {
                 'description': 'second key',
-                'readonly': False,
                 'required': True,
+                'updatable': False,
                 'type': 'Integer'
             },
         }
@@ -366,6 +319,127 @@ class TestPolicyBase(base.SenlinTestCase):
         res = policy._extract_policy_data(policy_data)
         self.assertIsNone(res)
 
+    @mock.patch.object(pb.Policy, '_build_conn_params')
+    @mock.patch('senlin.drivers.base.SenlinDriver')
+    def test_keystone(self, mock_sd, mock_params):
+        policy = self._create_policy('test-policy')
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        kc = mock.Mock()
+        driver = mock.Mock()
+        driver.identity.return_value = kc
+        mock_sd.return_value = driver
+
+        res = policy.keystone('user1', 'project1')
+
+        self.assertEqual(kc, res)
+        self.assertEqual(kc, policy._keystoneclient)
+        mock_params.assert_called_once_with('user1', 'project1')
+        mock_sd.assert_called_once_with()
+        driver.identity.assert_called_once_with(fake_params)
+
+    def test_keystone_already_initialized(self):
+        policy = self._create_policy('test-policy')
+        x_keystone = mock.Mock()
+        policy._keystoneclient = x_keystone
+
+        result = policy.keystone('foo', 'bar')
+
+        self.assertEqual(x_keystone, result)
+
+    @mock.patch.object(pb.Policy, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_nova(self, mock_driver, mock_params):
+        policy = self._create_policy('test-policy')
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        x_driver = mock.Mock()
+        mock_driver.return_value = x_driver
+
+        result = policy.nova('user1', 'project1')
+
+        x_nova = x_driver.compute.return_value
+        self.assertEqual(x_nova, result)
+        self.assertEqual(x_nova, policy._novaclient)
+        mock_params.assert_called_once_with('user1', 'project1')
+        x_driver.compute.assert_called_once_with(fake_params)
+
+    def test_nova_already_initialized(self):
+        policy = self._create_policy('test-policy')
+        x_nova = mock.Mock()
+        policy._novaclient = x_nova
+
+        result = policy.nova('foo', 'bar')
+
+        self.assertEqual(x_nova, result)
+
+    @mock.patch.object(pb.Policy, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_network(self, mock_driver, mock_params):
+        policy = self._create_policy('test-policy')
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        x_driver = mock.Mock()
+        mock_driver.return_value = x_driver
+
+        result = policy.network('user1', 'project1')
+
+        x_network = x_driver.network.return_value
+        self.assertEqual(x_network, result)
+        self.assertEqual(x_network, policy._networkclient)
+        mock_params.assert_called_once_with('user1', 'project1')
+        x_driver.network.assert_called_once_with(fake_params)
+
+    def test_network_already_initialized(self):
+        policy = self._create_policy('test-policy')
+        x_network = mock.Mock()
+        policy._networkclient = x_network
+
+        result = policy.network('foo', 'bar')
+
+        self.assertEqual(x_network, result)
+
+    @mock.patch.object(pb.Policy, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_lbaas(self, mock_driver, mock_params):
+        policy = self._create_policy('test-policy')
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        x_driver = mock.Mock()
+        mock_driver.return_value = x_driver
+
+        result = policy.lbaas('user1', 'project1')
+
+        x_lbaas = x_driver.loadbalancing.return_value
+        self.assertEqual(x_lbaas, result)
+        self.assertEqual(x_lbaas, policy._lbaasclient)
+        mock_params.assert_called_once_with('user1', 'project1')
+        x_driver.loadbalancing.assert_called_once_with(fake_params)
+
+    def test_lbaas_already_initialized(self):
+        policy = self._create_policy('test-policy')
+        x_lbaas = mock.Mock()
+        policy._lbaasclient = x_lbaas
+
+        result = policy.lbaas('foo', 'bar')
+
+        self.assertEqual(x_lbaas, result)
+
+    def test_default_need_check(self):
+        action = mock.Mock()
+        action.action = consts.CLUSTER_SCALE_IN
+        action.data = {}
+
+        policy = self._create_policy('test-policy')
+        res = policy.need_check('BEFORE', action)
+        self.assertTrue(res)
+
+        setattr(policy, 'TARGET', [('BEFORE', consts.CLUSTER_SCALE_IN)])
+        res = policy.need_check('BEFORE', action)
+        self.assertTrue(res)
+        res = policy.need_check('AFTER', action)
+        self.assertFalse(res)
+
     def test_default_pre_op(self):
         policy = self._create_policy('test-policy')
         res = policy.pre_op('CLUSTER_ID', 'FOO')
@@ -406,10 +480,10 @@ class TestPolicyBase(base.SenlinTestCase):
         res = policy.detach(cluster)
         self.assertEqual((True, None), res)
 
-    @mock.patch.object(db_api, 'cred_get')
-    @mock.patch.object(senlin_ctx, 'get_service_context')
+    @mock.patch.object(co.Credential, 'get')
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test_build_conn_params(self, mock_get_current, mock_get_service_ctx,
+    def test_build_conn_params(self, mock_get_current, mock_get_service_creds,
                                mock_cred_get):
         service_cred = {
             'auth_url': 'AUTH_URL',
@@ -429,33 +503,30 @@ class TestPolicyBase(base.SenlinTestCase):
             }
         }
 
-        cluster = mock.Mock()
-        cluster.user = 'user1'
-        cluster.project = 'project1'
-        cred = mock.Mock()
-        cred.cred = cred_info
-        mock_get_service_ctx.return_value = service_cred
+        cred = mock.Mock(cred=cred_info)
+        mock_get_service_creds.return_value = service_cred
         mock_get_current.return_value = current_ctx
         mock_cred_get.return_value = cred
-
         policy = self._create_policy('test-policy')
+
+        res = policy._build_conn_params('user1', 'project1')
+
         expected_result = {
             'auth_url': 'AUTH_URL',
             'username': 'senlin',
             'user_domain_name': 'default',
             'password': '123',
-            'trust_id': ['TRUST_ID']
+            'trust_id': 'TRUST_ID'
         }
-        res = policy._build_conn_params(cluster)
         self.assertEqual(expected_result, res)
-        mock_get_service_ctx.assert_called_once_with()
+        mock_get_service_creds.assert_called_once_with()
         mock_cred_get.assert_called_once_with(current_ctx, 'user1', 'project1')
 
-    @mock.patch.object(db_api, 'cred_get')
-    @mock.patch.object(senlin_ctx, 'get_service_context')
+    @mock.patch.object(co.Credential, 'get')
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
     def test_build_conn_params_trust_not_found(
-            self, mock_get_current, mock_get_service_ctx, mock_cred_get):
+            self, mock_get_current, mock_get_service_creds, mock_cred_get):
 
         service_cred = {
             'auth_url': 'AUTH_URL',
@@ -464,14 +535,13 @@ class TestPolicyBase(base.SenlinTestCase):
             'password': '123'
         }
 
-        mock_get_service_ctx.return_value = service_cred
+        mock_get_service_creds.return_value = service_cred
         mock_cred_get.return_value = None
-        cluster = mock.Mock()
-        cluster.user = 'user1'
-        cluster.project = 'project1'
-
         policy = self._create_policy('test-policy')
+
         ex = self.assertRaises(exception.TrustNotFound,
-                               policy._build_conn_params, cluster)
-        msg = "The trust for trustor (user1) could not be found."
+                               policy._build_conn_params,
+                               'user1', 'project1')
+
+        msg = "The trust for trustor 'user1' could not be found."
         self.assertEqual(msg, six.text_type(ex))
