@@ -10,58 +10,35 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from cryptography import fernet
+import datetime
+import mock
+from oslo_log import log as logging
+from oslo_utils import timeutils
 import requests
-from requests import exceptions
 import six
 
 from oslo_config import cfg
 
 from senlin.common import exception
 from senlin.common import utils
+from senlin.objects import service as service_obj
 from senlin.tests.unit.common import base
 
 
-class TestParameterParsing(base.SenlinTestCase):
-    def test_parse_bool(self):
-        name = 'param'
-        for value in ('True', 'true', 'TRUE', True):
-            self.assertTrue(utils.parse_bool_param(name, value))
-        for value in ('False', 'false', 'FALSE', False):
-            self.assertFalse(utils.parse_bool_param(name, value))
-        for value in ('foo', 't', 'f', 'yes', 'no', 'y', 'n', '1', '0', None):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_bool_param, name, value)
+class TestGetPositiveInt(base.SenlinTestCase):
 
-    def test_parse_int(self):
-        name = 'param'
-        cases = {0: 0, 2: 2, '0': 0, '2': 2}
+    def test_get_positive_int(self):
+        cases = {1: 1, 2: 2, '1': 1, '2': 2}
         for value, expected in cases.items():
-            actual = utils.parse_int_param(name, value)
+            res, actual = utils.get_positive_int(value)
+            self.assertTrue(res)
             self.assertEqual(expected, actual)
 
-        # A None should be returned directly
-        actual = utils.parse_int_param(name, None)
-        self.assertIsNone(actual)
-
-        for value in (2, '2'):
-            self.assertTrue(utils.parse_int_param(name, value,
-                                                  allow_zero=False))
-        for value in (0, '0'):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value,
-                              allow_zero=False)
-        for value in (-1, '-2'):
-            self.assertTrue(utils.parse_int_param(name, value,
-                                                  allow_negative=True))
-        for value in (-1, '-2'):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value)
-
-        for value in (1, 6):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value,
-                              lower_limit=2, upper_limit=5)
+        bad_values = ['foo', {}, [], -1, 1.5, 0.2, None]
+        for value in bad_values:
+            res, actual = utils.get_positive_int(value)
+            self.assertFalse(res)
+            self.assertEqual(0, actual)
 
 
 class Response(object):
@@ -78,9 +55,6 @@ class Response(object):
 
 
 class UrlFetchTest(base.SenlinTestCase):
-    def setUp(self):
-        super(UrlFetchTest, self).setUp()
-
     def test_file_scheme_default_behaviour(self):
         self.assertRaises(utils.URLFetchError,
                           utils.url_fetch, 'file:///etc/profile')
@@ -118,13 +92,15 @@ class UrlFetchTest(base.SenlinTestCase):
     def test_http_error(self):
         url = 'http://example.com/somedata'
 
-        self.patchobject(requests, 'get', side_effect=exceptions.HTTPError())
+        self.patchobject(requests, 'get',
+                         side_effect=requests.exceptions.HTTPError())
         self.assertRaises(utils.URLFetchError, utils.url_fetch, url)
 
     def test_non_exist_url(self):
         url = 'http://non-exist.com/somedata'
 
-        self.patchobject(requests, 'get', side_effect=exceptions.Timeout())
+        self.patchobject(requests, 'get',
+                         side_effect=requests.exceptions.Timeout())
         self.assertRaises(utils.URLFetchError, utils.url_fetch, url)
 
     def test_garbage(self):
@@ -133,52 +109,20 @@ class UrlFetchTest(base.SenlinTestCase):
     def test_max_fetch_size_okay(self):
         url = 'http://example.com/somedata'
         data = '{ "foo": "bar" }'
-        cfg.CONF.set_override('max_response_size', 500)
+        cfg.CONF.set_override('max_response_size', 500, enforce_type=True)
         self.patchobject(requests, 'get', return_value=Response(data))
         utils.url_fetch(url)
 
     def test_max_fetch_size_error(self):
         url = 'http://example.com/somedata'
         data = '{ "foo": "bar" }'
-        cfg.CONF.set_override('max_response_size', 5)
+        cfg.CONF.set_override('max_response_size', 5, enforce_type=True)
         self.patchobject(requests, 'get', return_value=Response(data))
         exception = self.assertRaises(utils.URLFetchError,
                                       utils.url_fetch, url)
         self.assertIn("Data exceeds", six.text_type(exception))
 
 
-class TestEncrypt(base.SenlinTestCase):
-    def test_encrypt(self):
-        msg = 'test-string'
-        msg_encrypted, key = utils.encrypt(msg)
-
-        self.assertIsInstance(msg_encrypted, six.string_types)
-        self.assertIsInstance(key, six.string_types)
-
-    def test_decrypt(self):
-        msg = 'test-string'
-        msg_encrypted, key = utils.encrypt(msg)
-
-        msg_decrypted = utils.decrypt(msg_encrypted, key)
-        self.assertEqual(msg, msg_decrypted)
-
-    def test_decrypt_invalid_key_msg(self):
-        msg = 'test-string'
-        msg_encrypted, key = utils.encrypt(msg)
-
-        invalid_key = 'fake-key'
-        self.assertRaises(fernet.InvalidToken, utils.decrypt,
-                          msg_encrypted, invalid_key)
-
-        invalid_msg = 'fake-msg'
-        self.assertRaises(ValueError, utils.decrypt,
-                          invalid_msg, key)
-
-        invalid_msg = fernet.Fernet.generate_key()
-        self.assertRaises(fernet.InvalidToken, utils.decrypt,
-                          invalid_msg, key)
-
-
 class TestRandomName(base.SenlinTestCase):
 
     def test_default(self):
@@ -209,3 +153,90 @@ class TestRandomName(base.SenlinTestCase):
 
         result = utils.random_name(-9)
         self.assertEqual('', result)
+
+
+class TestParseLevelValues(base.SenlinTestCase):
+
+    def test_none(self):
+        res = utils.parse_level_values(None)
+        self.assertIsNone(res)
+
+    def test_empty_list(self):
+        res = utils.parse_level_values([])
+        self.assertIsNone(res)
+
+    def test_single_value(self):
+        res = utils.parse_level_values('ERROR')
+        self.assertEqual([logging.ERROR], res)
+
+    def test_multi_values(self):
+        res = utils.parse_level_values(['WARNING', 'ERROR'])
+        self.assertEqual([logging.WARNING, logging.ERROR], res)
+
+    def test_with_invalid_values(self):
+        res = utils.parse_level_values(['warn', 'ERROR'])
+        self.assertEqual([logging.ERROR], res)
+
+    def test_with_integers(self):
+        res = utils.parse_level_values(40)
+        self.assertEqual([40], res)
+
+    def test_with_only_invalid_values(self):
+        res = utils.parse_level_values(['warn'])
+        self.assertIsNone(res)
+
+
+class TestGetPathParser(base.SenlinTestCase):
+
+    def test_normal(self):
+        res = utils.get_path_parser('foo.bar')
+        self.assertIsNotNone(res)
+
+    def test_bad_path(self):
+        err = self.assertRaises(exception.BadRequest,
+                                utils.get_path_parser,
+                                '^foo.bar')
+        self.assertEqual("Invalid attribute path - Unexpected "
+                         "character: ^.", six.text_type(err))
+
+
+class EngineDeathTest(base.SenlinTestCase):
+
+    def setUp(self):
+        super(EngineDeathTest, self).setUp()
+        self.ctx = mock.Mock()
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_none(self, mock_service):
+        mock_service.return_value = None
+        self.assertTrue(utils.is_engine_dead(self.ctx, 'fake_engine_id'))
+        mock_service.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_dead(self, mock_service):
+        delta = datetime.timedelta(seconds=3 * cfg.CONF.periodic_interval)
+        update_time = timeutils.utcnow(True) - delta
+        mock_service.return_value = mock.Mock(updated_at=update_time)
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id')
+
+        self.assertTrue(res)
+        mock_service.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_alive(self, mock_svc):
+        mock_svc.return_value = mock.Mock(updated_at=timeutils.utcnow(True))
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id')
+
+        self.assertFalse(res)
+        mock_svc.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_use_specified_duration(self, mock_svc):
+        mock_svc.return_value = mock.Mock(updated_at=timeutils.utcnow(True))
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id', 10000)
+
+        self.assertFalse(res)
+        mock_svc.assert_called_once_with(self.ctx, 'fake_engine_id')
